/****************************************************************\

  LSWeather by MrJukes <mrjukes@purdue.edu>
  Continued by Seth McCarus <lrover@mindspring.com>

  5.02, 6 Jun 2002
  ================
  - ReRedid GetWeather() to work (another change at
    wunderground?)
  - Began using URLDownloadToFile().  Initial testing indicates
    that this function is much more reliable than the fractured
    reading bit-by-bit of the web page (fewer incomplete
    readings!)

  5.01, 29 May 2002
  =================
  - Redid GetWeather() a bit to fix change at wunderground

  Version B5 by Seth McCarus
  August/September 2001

  Features added for LSWeather B5:
  - Window transparency
  - Calculates windchill
  - Calculates heat index
  - Time & date on conditions report
  - Records minimum and maximum temperatures
  - Option to mirror string formats in report conditions dialog
  - Separate format string for calm wind (not in config dialog)
  - Fixed bug in reading wind speed from international stations
  - ASCII characters in format strings: use '#xxx' for ASCII xxx
  - MAJOR addition: choice of absolute/relative feature display

  Issues:
  - Max and min temps are invalid when locale changes
  - Reports local time for distant locations in report dialog
  - Does not support multiple leading relative-placed features

\****************************************************************/


#include <time.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <windows.h>
#include <wininet.h>
#include <commctrl.h>
#include "lsweather.h"
#include "resource.h"
#include "lsapi.h"


/*** Global Variables***/
HWND hMainWnd;           // main window handle
HWND parent;             // parent window
wharfDataType wharfData; // Setup data passed to the DLL
int wndSize;             // Size of the wharf client
HINSTANCE Instance;
HMENU Popup;
HMENU FavPopup;
HBITMAP BGBMP = NULL;
HANDLE thread = NULL;
DWORD threadID = 0;

BOOL bErr = FALSE;
BOOL bOnTop = FALSE;
BOOL bActive = TRUE;
BOOL bHidden = FALSE;
BOOL bLoaded = FALSE;
BOOL bAbsolute = TRUE;
BOOL bChecking = FALSE;
BOOL bLoadModule = FALSE;
BOOL bCustomFormat = FALSE;

char ini[MAX_PATH] = "";
char thm[MAX_PATH] = "";
char szAppName[] = "LSWeather";
char szImageDir[MAX_PATH] = "";
char szFontName[MAX_PATH] = "System";
char szReportString[256] = "";
char szOrder[NUM_FEATURES + 1] = "";
char szDoubleClickAction[30] = "";
char g_szBaseURL[512];

int nCurrentHelp = 0;
int eUseMethod = useZip;
int nLastFav = 0;
int nTimer = 60000;
int nFontSize = MAX_FAVS;
int XPos, WndWidth,  ScreenX;;
int YPos, WndHeight, ScreenY;
int R, G, B, CR, CG, CB;
int nStartX, nStartY;
int nWindStyle;

char szFeature[][NUM_FEATURES] =
{
  "Temperature", "Humidity", "Dewpoint", "HeatIndex", "Wind",
    "Windchill", "Visibility", "Pressure", "MinTemp", "MaxTemp",
    "Sunrise", "Sunset", "Moonrise", "Moonset", "Conditions"
};


/*** Unit Conversion Functions ***/
int   FTOC(int F)   { return (F-32) * 5 / 9;    }
int   FTOK(int F)   { return FTOC(F) + 273;     }
int   MTOK(int M)   { return (int)(M * 1.6093); }
float ITOH(float I) { return (float)(I * 32.8); }


/*** Initialization Functions ***/

int initModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType* wd)
{
  return initModuleEx (ParentWnd, dllInst, wd->lsPath);
}


int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
  WNDCLASS wc;

  bLoadModule = TRUE;
  parent = ParentWnd; // Save parent window
  Instance = dllInst;
  sprintf(ini, "%s\\modules.ini", szPath);

  // Register the Window class

  memset(&wc, 0, sizeof(wc));
  wc.lpfnWndProc = WndProc;      // our window procedure
  wc.hInstance = dllInst;        // hInstance of DLL
  wc.lpszClassName = szAppName;  // our window class name
  wc.style = CS_DBLCLKS;

  if (!RegisterClass(&wc))
  {
    MessageBox(parent,"Error registering window class",szAppName, MB_OK);
    return 1;
  }

  LoadSetup();

  hMainWnd = CreateWindowEx(
              WS_EX_TOOLWINDOW,     // exstyles
              szAppName,            // our window class name
              NULL,                 // no title means no task list entry (thanks Blkhawk!)
              WS_POPUP,             // window style
              XPos, YPos,           // position
              WndWidth, WndHeight,  // width & height of window
              GetDesktopWindow(),   // parent window (litestep wharf window)
              NULL,                 // no menu
              dllInst,              // hInstance of DLL
              0);                   // no window creation data

  if (!hMainWnd)
  {
    MessageBox(NULL,"Error creating window",szAppName,MB_OK);
    return 1;
  }

  //this part added for window (not alpha) transparency
  if (BGBMP) SetWindowBitmapRgn(hMainWnd, BGBMP);

  //set magicDWord - make it a 'Litestep window'
  SetWindowLong(hMainWnd, GWL_USERDATA, magicDWord);

  //show the window
  ShowWindow(hMainWnd, SW_SHOWNORMAL);

  thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)GetWeather, (LPVOID)NULL, 0, &threadID);
  SetTimer(hMainWnd, 0, nTimer, NULL);

  return 0;
}


/*** SetWindowBitmapRgn ***/
void SetWindowBitmapRgn(HWND hwnd, HBITMAP bmp)
{
  if (hwnd && bmp)
  {
    int xPos=0, yPos=0;
    HDC hdc = CreateCompatibleDC(NULL);
    HRGN hTransRgn=NULL;
    HRGN hMainRgn=NULL;
    BITMAP bitmap;

    GetObject(bmp, sizeof(bitmap), &bitmap);
    SelectObject(hdc, bmp);
    hMainRgn = CreateRectRgn(0, 0, bitmap.bmWidth, bitmap.bmHeight);

    for (yPos=0; yPos < bitmap.bmHeight; xPos=0, yPos++)
    {
      for (xPos=0; xPos < bitmap.bmWidth; xPos++)
      {
        COLORREF c = GetPixel(hdc, xPos, yPos);
        if (c == 0x00FF00FF)
        {
          HRGN hTempRgn = CreateRectRgn(xPos, yPos, xPos+1, yPos+1);
          if (!hTransRgn) hTransRgn = CreateRectRgn(xPos, yPos, xPos+1, yPos+1);
          CombineRgn(hTransRgn, hTransRgn, hTempRgn, RGN_OR);
          DeleteObject(hTempRgn);
        }
      }
    }

    CombineRgn(hMainRgn, hMainRgn, hTransRgn, RGN_DIFF);
    SetWindowRgn(hwnd, hMainRgn, FALSE);

    DeleteDC(hdc);
    DeleteObject(hTransRgn);
    DeleteObject(hMainRgn);
  }
  else
  {
    RECT r;
    HRGN rgn;
    GetClientRect(hwnd, &r);
    rgn = CreateRectRgn(r.left, r.top, r.right, r.bottom);
    SetWindowRgn(hwnd, rgn, FALSE);
  }
}


/*******************/
/* DLL Entry point */
/*******************/
int initWharfModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType *wd)
{
  parent = ParentWnd; // Save parent window

  // Duplicate wharfData since the one we get will be destroyed
  memcpy(&wharfData, wd, sizeof(wharfDataType));
  if (wharfData.borderSize < 0)
    wharfData.borderSize = 0;
  wndSize = 64 - wharfData.borderSize * 2;
  Instance = dllInst;

  {
    // Register the Window class
    WNDCLASS wc;

    memset(&wc,0,sizeof(wc));
    wc.lpfnWndProc = WndProc;      // our window procedure
    wc.hInstance = dllInst;        // hInstance of DLL
    wc.lpszClassName = szAppName;  // our window class name
    wc.style = CS_DBLCLKS;

    if (!RegisterClass(&wc))
    {
      MessageBox(parent,"Error registering window class",szAppName, MB_OK);
      return 1;
    }
  }

  LoadSetup();

  hMainWnd = CreateWindowEx(
        0,                  // exstyles
        szAppName,          // our window class name
        szAppName,          // use description for a window title
        WS_CHILD,           // window style
        wharfData.borderSize, wharfData.borderSize, // position
        wndSize, wndSize,   // width & height of window
        parent,             // parent window (litestep wharf window)
        NULL,               // no menu
        dllInst,            // hInstance of DLL
        0);                 // no window creation data

  if (!hMainWnd)
  {
      MessageBox(parent,"Error creating window",szAppName,MB_OK);
      return 1;
  }

  SetWindowLong(hMainWnd,GWL_USERDATA,magicDWord);
  ShowWindow(hMainWnd,SW_SHOWNORMAL);

  thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)GetWeather, (LPVOID)NULL, 0, &threadID);
  SetTimer(hMainWnd, 0, nTimer, NULL);

  return 0;
}

void quitModule(HINSTANCE dllInst)
{
  quitWharfModule(dllInst);
}


/***********************/
/* DLL closeup request */
/***********************/
void quitWharfModule(HINSTANCE dllInst)
{
//  RegisterServiceProcess(NULL, 0); //unregister
  TerminateThread(thread, 0);
  KillTimer(hMainWnd, 0);
  DestroyMenu(Popup);
  DestroyMenu(FavPopup);
  DeleteObject(BGBMP);
  DestroyWindow(hMainWnd);             // delete our window
  UnregisterClass(szAppName, dllInst); // unregister window class
}


/*** LoadSetup ***/
void LoadSetup()
{
  char temp[256] = "";
  int i;

  ScreenX = GetSystemMetrics(SM_CXSCREEN);
  ScreenY = GetSystemMetrics(SM_CYSCREEN);

  for (i = 0; i < NUM_FEATURES; i++)
  {
    feature[i].bUsed = FALSE;
    strcpy(feature[i].szName, szFeature[i]);
    strcpy(feature[i].szSel, NOSEL);
  }

  AddBangCommand("!LSWHide",BangHide);
  AddBangCommand("!LSWShow",BangShow);
  AddBangCommand("!LSWToggle",BangToggle);
  AddBangCommand("!LSWUpdate",BangUpdate);
  AddBangCommand("!LSWReport",BangReport);
  AddBangCommand("!LSWChangeSettings",BangChangeSettings);
  AddBangCommand("!LSWLaunchWebpage",BangLaunchWebpage);
  AddBangCommand("!LSWAlwaysOnTop",BangAlwaysOnTop);

  // Make sure we know where the modules.ini is
  if (!bLoadModule) sprintf(ini, "%smodules.ini", wharfData.lsPath);

  // Find the theme file we are to use
  GetPrivateProfileString(szAppName, "ThemeFile", "NO THEME HONKY", thm, MAX_PATH, ini);
  if (!strcmp(thm, "NO THEME HONKY"))
  {
    MessageBox(parent, "Thank you for trying out LSWeather.  It seems as though you do not have a theme file specified in your modules.ini.  Please hit \"Ok\" and configure LSWeather to your liking.", szAppName, MB);
    !bLoadModule ? sprintf(thm, "%slsweather.thm", wharfData.lsPath) : sprintf(thm, "c:\\litestep\\lsweather.thm");
    DialogBox(Instance, MAKEINTRESOURCE(IDD_CONFIG), parent, ConfigDialogProc);
  }

  // The modules.ini only holds XY coords, Location Info, DisplayType, Timer info, Usage Type, Active Toggle. and Favorites
  GetPrivateProfileString(szAppName, "ZipCode", "47906", loc.zip, 6, ini);
  GetPrivateProfileString(szAppName, "City", "Lafayette", loc.city, 50, ini);
  GetPrivateProfileString(szAppName, "State", "IN", loc.state, 5, ini);
  GetPrivateProfileString(szAppName, "GlobalStation", "", loc.station, 15, ini);
  GetPrivateProfileString("Main", "BaseURL",  "http://www.wunderground.com/", g_szBaseURL, sizeof g_szBaseURL, ini);

  XPos = GetPrivateProfileInt(szAppName, "DesktopX", 50, ini);
  YPos = GetPrivateProfileInt(szAppName, "DesktopY", 50, ini);
  nTimer = GetPrivateProfileInt(szAppName, "Timer", 1, ini) * 60000;
  eUseMethod = GetPrivateProfileInt(szAppName, "UseMethod", useZip, ini);
  bActive = GetPrivateProfileInt(szAppName, "Active", 1, ini);

  LoadThemeInfo();

  FavPopup = CreatePopupMenu();
  for (i=0; i<MAX_FAVS; i++)
  {
    char name[256] = "";

    sprintf(temp, "Favorite%d", i);
    GetPrivateProfileString(szAppName, temp, "", name, 256, ini);

    if (strlen(name))
    {
      AppendMenu(FavPopup, MF_ENABLED | MF_STRING, 200 + i, name);
      nLastFav++;
    }
    else break;
  }

  Popup = CreatePopupMenu();
  AppendMenu(Popup, MF_ENABLED | MF_STRING, M_UPDATE, "&Update");
  AppendMenu(Popup, MF_ENABLED | MF_STRING, M_ACTIVE, "&Active");
  AppendMenu(Popup, MF_ENABLED | MF_STRING, M_REPORT, "&Report Current Conditions");
  AppendMenu(Popup, MF_ENABLED | MF_STRING, M_SETTINGS, "Change &Settings");
  AppendMenu(Popup, MF_ENABLED | MF_STRING, M_LAUNCH, "Launch &Webpage");
  AppendMenu(Popup, MF_ENABLED | MF_STRING, M_ABOUT, "About &LSWeather");
  AppendMenu(Popup, MF_ENABLED | MF_STRING, M_RESET, "Reset &Min/Max Temps");
  AppendMenu(Popup, MF_SEPARATOR, 0, "");
  AppendMenu(Popup, MF_ENABLED | MF_STRING | MF_POPUP, (int)FavPopup, "Favorites");
  AppendMenu(Popup, MF_SEPARATOR, 0, "");
  AppendMenu(Popup, MF_ENABLED | MF_STRING, M_USEZIP, "Use &Zip Code");
  AppendMenu(Popup, MF_ENABLED | MF_STRING, M_USECITY, "Use Cit&y");
  AppendMenu(Popup, MF_ENABLED | MF_STRING, M_USESTATION, "Use &Global Station");

  bActive ? CheckMenuItem(Popup, M_ACTIVE, MF_CHECKED) : CheckMenuItem(Popup, M_ACTIVE, MF_CHECKED);

  if (eUseMethod == useZip) CheckMenuItem(Popup, M_USEZIP, MF_CHECKED);
  else if (eUseMethod == useCityState) CheckMenuItem(Popup, M_USECITY, MF_CHECKED);
  else if (eUseMethod == useStation) CheckMenuItem(Popup, M_USESTATION, MF_CHECKED);
}


/*** LoadThemeInfo ***/
void LoadThemeInfo()
{
  char rgb[15] = "";
  char bg[MAX_PATH] = "";

  if (bLoadModule)
  {
    WndWidth = GetPrivateProfileInt("Main", "Width", 64, thm);
    WndHeight = GetPrivateProfileInt("Main", "Height", 64, thm);
  }
  else
  {
    WndWidth = WndHeight = wndSize;
  }

  GetPrivateProfileString("Main", "ImageDir", "", szImageDir, MAX_PATH, thm);
  if (szImageDir[strlen(szImageDir)-1] != '\\') strcat(szImageDir, "\\");
  GetPrivateProfileString("Main", "BackGroundPix", "", bg, MAX_PATH, thm);
  if (bg[0])
  {
    char temp[256] = "";
    sprintf(temp, "%s%s", szImageDir, bg);
    BGBMP = LoadImage(Instance, temp, IMAGE_BITMAP, WndWidth, WndHeight, LR_LOADFROMFILE);
  }

  feature[pres].bDisplay = GetPrivateProfileInt("Main", "DisplayPressure", 0, thm);
  feature[wind].bDisplay = GetPrivateProfileInt("Main", "DisplayWind", 0, thm);
  feature[tmp].bDisplay = GetPrivateProfileInt("Main", "DisplayTemperature", 0, thm);
  feature[hum].bDisplay = GetPrivateProfileInt("Main", "DisplayHumidity", 0, thm);
  feature[dew].bDisplay = GetPrivateProfileInt("Main", "DisplayDewpoint", 0, thm);
  feature[vis].bDisplay = GetPrivateProfileInt("Main", "DisplayVisibility", 0, thm);
  feature[min].bDisplay = GetPrivateProfileInt("Main", "DisplayMinTemp", 0, thm);
  feature[max].bDisplay = GetPrivateProfileInt("Main", "DisplayMaxTemp", 0, thm);
  feature[sr].bDisplay = GetPrivateProfileInt("Main", "DisplaySunrise", 0, thm);
  feature[ss].bDisplay = GetPrivateProfileInt("Main", "DisplaySunset", 0, thm);
  feature[mr].bDisplay = GetPrivateProfileInt("Main", "DisplayMoonrise", 0, thm);
  feature[ms].bDisplay = GetPrivateProfileInt("Main", "DisplayMoonset", 0, thm);
  feature[wc].bDisplay = GetPrivateProfileInt("Main", "DisplayWindchill", 0, thm);
  feature[hi].bDisplay = GetPrivateProfileInt("Main", "DisplayHeatIndex", 0, thm);
  feature[cond].bDisplay = GetPrivateProfileInt("Conditions", "DisplayText", 0, thm);

  //get relative placement strings
  GetPrivateProfileString("Temperature", "Follows", NOSEL, feature[tmp].szSel, 20, thm);
  GetPrivateProfileString("Humidity",    "Follows", NOSEL, feature[hum].szSel, 20, thm);
  GetPrivateProfileString("Dewpoint",    "Follows", NOSEL, feature[dew].szSel, 20, thm);
  GetPrivateProfileString("HeatIndex",   "Follows", NOSEL, feature[hi].szSel, 20, thm);
  GetPrivateProfileString("Wind",        "Follows", NOSEL, feature[wind].szSel, 20, thm);
  GetPrivateProfileString("Windchill",   "Follows", NOSEL, feature[wc].szSel, 20, thm);
  GetPrivateProfileString("Visibility",  "Follows", NOSEL, feature[vis].szSel, 20, thm);
  GetPrivateProfileString("Pressure",    "Follows", NOSEL, feature[pres].szSel, 20, thm);
  GetPrivateProfileString("MinTemp",     "Follows", NOSEL, feature[min].szSel, 20, thm);
  GetPrivateProfileString("MaxTemp",     "Follows", NOSEL, feature[max].szSel, 20, thm);
  GetPrivateProfileString("Sunrise",     "Follows", NOSEL, feature[sr].szSel, 20, thm);
  GetPrivateProfileString("Sunset",      "Follows", NOSEL, feature[ss].szSel, 20, thm);
  GetPrivateProfileString("Moonrise",    "Follows", NOSEL, feature[mr].szSel, 20, thm);
  GetPrivateProfileString("Moonset",     "Follows", NOSEL, feature[ms].szSel, 20, thm);
  GetPrivateProfileString("Conditions",  "Follows", NOSEL, feature[cond].szSel, 20, thm);

  GetPrivateProfileString("Main", "FontRGB", "255 255 255", rgb, 15, thm);
  R = atoi(strtok(rgb, " "));
  G = atoi(strtok(NULL, " "));
  B = atoi(strtok(NULL, ""));

  memset(&rgb, 0, sizeof(rgb));
  GetPrivateProfileString("Main", "FontCheckRGB", "255 0 0", rgb, 15, thm);
  CR = atoi(strtok(rgb, " "));
  CG = atoi(strtok(NULL, " "));
  CB = atoi(strtok(NULL, ""));

  nFontSize = GetPrivateProfileInt("Main", "FontSize", 10, thm);
  GetPrivateProfileString("Main", "Font", "System", szFontName, MAX_PATH, thm);
  GetPrivateProfileString("Main", "DoubleClick", "Report Conditions", szDoubleClickAction, 30, thm);
  nWindStyle = GetPrivateProfileInt("Wind", "WindStyle", 1, thm);

  bCustomFormat  = GetPrivateProfileInt("Main", "CustomFormat", 0, thm);
  bAbsolute      = GetPrivateProfileInt("Main", "AbsolutePlacement", 1, thm);
  nStartX  = GetPrivateProfileInt("Main", "StartX", 0, thm);
  nStartY  = GetPrivateProfileInt("Main", "StartY", 0, thm);
  if (!bAbsolute) GetFeatureOrder();

  tempMax.day  = GetPrivateProfileInt("Main", "MaxDay",  -999, thm);
  tempMax.hour = GetPrivateProfileInt("Main", "MaxHour", -999, thm);
  tempMax.min  = GetPrivateProfileInt("Main", "MaxMin",  -999, thm);
  tempMax.temp = GetPrivateProfileInt("Main", "MaxTemp", -999, thm);
  tempMin.day  = GetPrivateProfileInt("Main", "MinDay",   999, thm);
  tempMin.hour = GetPrivateProfileInt("Main", "MinHour",  999, thm);
  tempMin.min  = GetPrivateProfileInt("Main", "MinMin",   999, thm);
  tempMin.temp = GetPrivateProfileInt("Main", "MinTemp",  999, thm);
}


/*********************************************************************/
/* Window procedure for our window           */
/*********************************************************************/
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch (message)
  {
    case WM_CREATE:
    case WM_ERASEBKGND:
    case WM_NCMOUSEMOVE:
    case WM_MOUSEMOVE:
      return 0;

    case WM_NCHITTEST:
      if (bLoadModule)
        return HTCAPTION;
      else
        return HTCLIENT;

    case WM_NCPAINT:
    case WM_PAINT:
    { __try  {
      PAINTSTRUCT ps;
      HDC hdc = BeginPaint(hwnd, &ps);
      HDC buf = CreateCompatibleDC(NULL);
      HDC src = CreateCompatibleDC(NULL);
      HBITMAP bufbmp = CreateCompatibleBitmap(hdc, WndWidth, WndHeight);
      HBITMAP oldbuf, oldsrc;
      HFONT font=NULL;
      char temp[256] = "";
      char out[256] = "";
      char holdme[256] = "";

      if (!BGBMP)
      {
        BGBMP = CreateCompatibleBitmap(hdc, WndWidth, WndHeight);
        oldbuf = (HBITMAP)SelectObject(buf, BGBMP);
        BitBlt(buf, 0, 0, WndWidth, WndHeight, hdc, 0, 0, SRCCOPY);
        SelectObject(buf, oldbuf);
      }

      oldbuf = (HBITMAP)SelectObject(buf, bufbmp);
      oldsrc = (HBITMAP)SelectObject(src, BGBMP);
      BitBlt(buf, 0, 0, WndWidth, WndHeight, src, 0, 0, SRCCOPY);

      font = CreateFont(nFontSize, 0, 0, 0, FW_DONTCARE, FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, szFontName);
      SelectObject(buf, font);

      bChecking ? SetTextColor(buf, RGB(CR, CG, CB)) : SetTextColor(buf, RGB(R, G, B));
      SetBkMode(buf, TRANSPARENT);

      if (GetPrivateProfileInt("Conditions", "DisplayBmp", 0, thm))
      {
        HDC chdc = CreateCompatibleDC(NULL);
        HBITMAP cbmp = NULL;
        char buffer[MAX_PATH] = "";
        char bmp[MAX_PATH] = "";

        if (strlen(current.conditions))
        {
          if (!bErr)
          {
            GetPrivateProfileString("Conditions", current.conditions, "NO FINDY", buffer, MAX_PATH, thm);
            if (!strcmp(buffer, "NO FINDY")) GetPrivateProfileString("Conditions", "Unknown", "lsweather.bmp", buffer, MAX_PATH, thm);
          }
          else
          {
            GetPrivateProfileString("Conditions", "Error", "NO FINDY", buffer, MAX_PATH, thm);
            if (!strcmp(buffer, "NO FINDY")) GetPrivateProfileString("Conditions", "Unknown", "lsweather.bmp", buffer, MAX_PATH, thm);
          }

          sprintf(bmp, "%s%s", szImageDir, buffer);
          cbmp = LoadImage(Instance, bmp, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);

          if (cbmp)
          {
            SelectObject(chdc, cbmp);
            BitBlt(buf,
               GetPrivateProfileInt("Conditions", "BmpX", 0, thm),
               GetPrivateProfileInt("Conditions", "BmpY", 0, thm),
               GetPrivateProfileInt("Conditions", "BmpWidth", 0, thm),
               GetPrivateProfileInt("Conditions", "BmpHeight", 0, thm),
               chdc, 0, 0, SRCCOPY);
          }
        }

        DeleteDC(chdc);
        DeleteObject(cbmp);
      }

      //save non-replaced conditions text
      strcpy(holdme, current.conditions);
      sprintf(out, "%s Text Replacement", current.conditions);
      GetPrivateProfileString("Conditions", out, "", temp, 256, thm);
      if (strlen(temp)) strcpy(current.conditions, temp);

      //display each item...
      if (bAbsolute)
      {
        //...in standard order
        int i;
        for (i = 0; i < NUM_FEATURES; i++)
          if (feature[i].bDisplay)
            DisplayValue(buf, szFeature[i]);
      }
      else
      {
        //...in the order specified by szOrder
        UINT i;
        for (i = 0; i < strlen(szOrder); i++)
          DisplayValue(buf, szFeature[szOrder[i]-65]);
      }

      //restore non-replaced conditions text
      strcpy(current.conditions, holdme);

      BitBlt(hdc, 0, 0, WndWidth, WndHeight, buf, 0, 0, SRCCOPY);

      SelectObject(src, oldsrc);
      DeleteDC(src);
      SelectObject(buf, oldbuf);
      DeleteDC(buf);
      DeleteObject(bufbmp);
      DeleteObject(font);
      EndPaint(hwnd, &ps);
      }__except(1) { bErr=TRUE; }
    }
    return 0;

    case WM_TIMER:
      thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)GetWeather, (LPVOID)NULL, 0, &threadID);
      return 0;

    case WM_WINDOWPOSCHANGING:
    {
      if (bLoadModule)
      {
        WINDOWPOS* pos = (WINDOWPOS*)lParam;
        char temp[15]= "";

        if (!(pos->flags & SWP_NOMOVE))
        {
          if (pos->x + WndWidth >= ScreenX - SNAP_DIST) pos->x = ScreenX - WndWidth;
          else if (pos->x <= SNAP_DIST) pos->x = 0;
          if (pos->y + WndHeight >= ScreenY - SNAP_DIST) pos->y = ScreenY - WndHeight;
          else if (pos->y <= SNAP_DIST) pos->y = 0;

          XPos = pos->x;
          YPos = pos->y;
        }

        sprintf(temp, "%d", XPos);
        WritePrivateProfileString(szAppName, "DesktopX", temp, ini);
        sprintf(temp, "%d", YPos);
        WritePrivateProfileString(szAppName, "DesktopY", temp, ini);
      }
    }
    return 0;

    case WM_KEYDOWN:
    case WM_KEYUP:
      PostMessage(parent, message, wParam, lParam);
      return 0;

    case WM_NCLBUTTONDBLCLK:
    case WM_LBUTTONDBLCLK:
      if (!strcmp(szDoubleClickAction, "Launch Webpage")) SendMessage(hwnd, WM_COMMAND, (WPARAM)M_LAUNCH, 0);
      else if (!strcmp(szDoubleClickAction, "Change Settings")) SendMessage(hwnd, WM_COMMAND, (WPARAM)M_SETTINGS, 0);
      else if (!strcmp(szDoubleClickAction, "Report Conditions")) SendMessage(hwnd, WM_COMMAND, (WPARAM)M_REPORT, 0);
      else if (!strcmp(szDoubleClickAction, "Update")) SendMessage(hwnd, WM_COMMAND, (WPARAM)M_UPDATE, 0);
      return 0;

    case WM_NCRBUTTONUP:
    case WM_RBUTTONUP:
    {
      RECT r;
      GetWindowRect(hwnd, &r);
      PostMessage(GetParent(GetParent(parent)), 9182, r.top+(int)HIWORD(lParam), r.left+(int)LOWORD(lParam));
      return 0;
    }

    case WM_NCRBUTTONDOWN:
    case WM_RBUTTONDOWN:
    {
      DWORD dw = GetMessagePos();
      TrackPopupMenu(Popup, TPM_RIGHTALIGN | TPM_RIGHTBUTTON, LOWORD(dw), HIWORD(dw), 0, hMainWnd, NULL);
      return 0;
    }

    case WM_COMMAND:
    {
      switch (wParam)
      {
        case M_UPDATE: // Update
          thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)GetWeather, (LPVOID)NULL, 0, &threadID);
          break;

        case M_ACTIVE: // Active
        {
          if (bActive)
          {
            bActive=FALSE;
            CheckMenuItem(Popup, M_ACTIVE, MF_UNCHECKED);
            KillTimer(hwnd, 0);
          }
          else
          {
            bActive=TRUE;
            CheckMenuItem(Popup, M_ACTIVE, MF_CHECKED);
            thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)GetWeather, (LPVOID)NULL, 0, &threadID);
            SetTimer(hwnd, 0, nTimer, NULL);
          }
        }
        break;

        case M_SETTINGS: // Change Settings
          DialogBox(Instance, MAKEINTRESOURCE(IDD_CONFIG), hMainWnd, ConfigDialogProc);
          break;

        case M_LAUNCH: // Launch wunderground
        {
          char FULL_URL[256] = "";
          if (ConstructURL(FULL_URL))
            ShellExecute(NULL, "open", FULL_URL, NULL, NULL, SW_SHOWNORMAL);
        }
        break;

        case M_ABOUT: // About
          DialogBox(Instance, MAKEINTRESOURCE(IDD_ABOUTLSWEATHER), hMainWnd, AboutDialogProc);
          break;

        case M_RESET:
        {
          time_t now;
          struct tm* local;

          time(&now);
          local = localtime(&now); //military format
          tempMax.day  = local->tm_mday;
          tempMax.hour = local->tm_hour;
          tempMax.min  = local->tm_min;
          tempMax.temp = current.temp;
          WritePrivateProfileInt("Main", "MaxTemp", tempMax.temp, ini);
          WritePrivateProfileInt("Main", "MaxHour", tempMax.hour, ini);
          WritePrivateProfileInt("Main", "MaxMin",  tempMax.min, ini);
          WritePrivateProfileInt("Main", "MaxDay",  tempMax.day, ini);
          tempMin.day  = local->tm_mday;
          tempMin.hour = local->tm_hour;
          tempMin.min  = local->tm_min;
          tempMin.temp = current.temp;
          WritePrivateProfileInt("Main", "MinTemp", tempMin.temp, ini);
          WritePrivateProfileInt("Main", "MinHour", tempMin.hour, ini);
          WritePrivateProfileInt("Main", "MinMin",  tempMin.min, ini);
          WritePrivateProfileInt("Main", "MinDay",  tempMin.day, ini);
          break;
        }

        case M_USEZIP:
          eUseMethod = useZip;
          WritePrivateProfileString(szAppName, "UseMethod", "0", ini);
          CheckMenuItem(Popup, M_USEZIP, MF_CHECKED);
          CheckMenuItem(Popup, M_USECITY, MF_UNCHECKED);
          CheckMenuItem(Popup, M_USESTATION, MF_UNCHECKED);
          thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)GetWeather, (LPVOID)NULL, 0, &threadID);
          break;

        case M_USECITY:
          eUseMethod = useCityState;
          WritePrivateProfileString(szAppName, "UseMethod", "1", ini);
          CheckMenuItem(Popup, M_USEZIP, MF_UNCHECKED);
          CheckMenuItem(Popup, M_USECITY, MF_CHECKED);
          CheckMenuItem(Popup, M_USESTATION, MF_UNCHECKED);
          thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)GetWeather, (LPVOID)NULL, 0, &threadID);
          break;

        case M_USESTATION:
          eUseMethod = useStation;
          WritePrivateProfileString(szAppName, "UseMethod", "2", ini);
          CheckMenuItem(Popup, M_USEZIP, MF_UNCHECKED);
          CheckMenuItem(Popup, M_USECITY, MF_UNCHECKED);
          CheckMenuItem(Popup, M_USESTATION, MF_CHECKED);
          thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)GetWeather, (LPVOID)NULL, 0, &threadID);
          break;

        case M_REPORT:
          DialogBox(Instance, MAKEINTRESOURCE(IDD_REPORT), hMainWnd, ReportDialogProc);
          break;

        // The Favorites
        case 200: SetActiveCity(0); break;
        case 201: SetActiveCity(1); break;
        case 202: SetActiveCity(2); break;
        case 203: SetActiveCity(3); break;
        case 204: SetActiveCity(4); break;
        case 205: SetActiveCity(5); break;
        case 206: SetActiveCity(6); break;
        case 207: SetActiveCity(7); break;
        case 208: SetActiveCity(8); break;
        case 209: SetActiveCity(9); break;
      }
    }
    return 0;
  }
  return DefWindowProc(hwnd,message,wParam,lParam);
}


/*** ConfigDialogProc ***/
BOOL CALLBACK ConfigDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg)
  {
    case WM_INITDIALOG:
    {
      char st[5] = "";
      SetDlgItemText(hwnd, IDC_THEME_FILE, thm);
      SetupBitmapListView(hwnd);
      SetConfigDlgValues(hwnd);
      GetPrivateProfileString(szAppName, "State", "", st, 5, ini);
      SetupStateBox(hwnd, IDC_STATE, st);
      bLoaded=TRUE;

      //hack to fill list boxes with current selection
      {
        DWORD i;
        for (i = 0; i < NUM_FEATURES; i++)
        {
          SendMessage(GetDlgItem(hwnd, i + REL_START), CB_RESETCONTENT, 0, 0);
          SendMessage(GetDlgItem(hwnd, i + REL_START), CB_ADDSTRING,    0, (LPARAM)feature[i].szSel);
          SendMessage(GetDlgItem(hwnd, i + REL_START), CB_SELECTSTRING, 0, (LPARAM)feature[i].szSel);
        }
      }

      ShowControls(hwnd, !bAbsolute);
      return TRUE;
    }

    case WM_SYSCOMMAND:
      if (wParam == SC_CLOSE)
      {
        EndDialog(hwnd, 0);
        bLoaded=FALSE;
      }
      break;

    case WM_NOTIFY:
      return TRUE;

    case WM_HELP:
    {
      HELPINFO* hi = (LPHELPINFO)lParam;
      nCurrentHelp = hi->iCtrlId;
      DialogBox(Instance, MAKEINTRESOURCE(IDD_HELP), hwnd, HelpDialogProc);
      return TRUE;
    }

    case WM_COMMAND:
    {
      switch HIWORD(wParam)
      {
        case BN_CLICKED:
        {
          BOOL APPLY=FALSE;
          switch LOWORD(wParam)
          {
            case IDC_RELATIVE:  //hide absolute controls
              ShowControls(hwnd, TRUE);
              break;

            case IDC_ABSOLUTE:  //hide relative controls
              ShowControls(hwnd, FALSE);
              break;

            case IDC_APPLY: APPLY = TRUE;
            case IDOK:
              SaveDlgConfigValues(hwnd);
              SetWindowPos(hMainWnd, 0, 0, 0, WndWidth, WndHeight, SWP_NOSIZE | SWP_NOMOVE);

              KillTimer(hMainWnd, 0);
              //thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)GetWeather, (LPVOID)NULL, 0, &threadID);
              SetTimer(hMainWnd, 0, nTimer, NULL);

            case IDCANCEL:
              if (!APPLY)
              {
                EndDialog(hwnd, 0);
                bLoaded=FALSE;
              }
              break;

            case IDC_BITMAP_NEW:
              DialogBox(Instance, MAKEINTRESOURCE(IDD_ADD_CONDITION), hwnd, AddConditionDialogProc);
              break;

            case IDC_THEME_BROWSE:
              BrowseForFile(hwnd, IDC_THEME_FILE , "Theme Files (*.thm)\0*.thm\0All Files (*.*)\0*.*\0\0");
              break;

            case IDC_BITMAP_REMOVE:
            {
              HWND lv = GetDlgItem(hwnd, IDC_CONDITION_LIST);
              int index = ListView_GetSelectionMark(lv);
              char temp[256] = "";
              char temp2[256] = "";

              ListView_GetItemText(lv, index, 0, temp, 256);
              WritePrivateProfileString("Conditions", temp, NULL, thm);
              sprintf(temp2, "%s Text Replacement", temp);
              WritePrivateProfileString("Conditions", temp2, NULL, thm);
              ListView_DeleteItem(lv, index);
              break;
            }

            case IDC_USE_ZIPCODE:
              SendMessage(GetDlgItem(hwnd, IDC_USE_CITY_STATE), BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
              SendMessage(GetDlgItem(hwnd, IDC_USE_GLOBAL_STATION), BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
              break;

            case IDC_USE_CITY_STATE:
              SendMessage(GetDlgItem(hwnd, IDC_USE_ZIPCODE), BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
              SendMessage(GetDlgItem(hwnd, IDC_USE_GLOBAL_STATION), BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
              break;

            case IDC_USE_GLOBAL_STATION:
              SendMessage(GetDlgItem(hwnd, IDC_USE_ZIPCODE), BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
              SendMessage(GetDlgItem(hwnd, IDC_USE_CITY_STATE), BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
              break;

            case IDC_ADD_TO_FAVORITES:
              DialogBox(Instance, MAKEINTRESOURCE(IDD_ADD_FAVORITE), hwnd, AddFavoriteDialogProc);
              break;
          }
        }
        break;

        case CBN_DROPDOWN:
        {
          //clicked on a drop-down combo box... populate it
          int nID = LOWORD(wParam) - REL_START;
          if (nID >= 0 && nID <= NUM_FEATURES)
          {
            HWND hCombo = GetDlgItem(hwnd, LOWORD(wParam));
            int i;

            SendMessage(hCombo, CB_RESETCONTENT, 0, 0);
            SendMessage(hCombo, CB_ADDSTRING, 0, (LPARAM)NOSEL);
            for (i = 0; i < NUM_FEATURES; i++)
            {
              if (nID != i &&       //can't follow yourself!
                 (!feature[i].bUsed ||    //feature unused or...
                  !strcmp(feature[nID].szSel, feature[i].szName)))   //...used by this combo
                SendMessage(hCombo, CB_ADDSTRING, 0, (LPARAM)szFeature[i]);
            }
            //highlight selected string
            SendMessage(hCombo, CB_SETCURSEL, SendMessage(hCombo, CB_FINDSTRING, 0, (LPARAM)feature[nID].szSel), 0);
          }
        }
        break;

        case CBN_CLOSEUP:
        {
          int nID = LOWORD(wParam) - REL_START;
          if (nID >= 0 && nID <= NUM_FEATURES)
          {
            char newFeature[20] = "", oldFeature[20] = "";
            HWND hCombo = GetDlgItem(hwnd, LOWORD(wParam));
            int nSel = SendMessage(hCombo, CB_GETCURSEL, 0, 0);

            SendMessage(hCombo, CB_GETLBTEXT, nSel, (LPARAM)newFeature);
            strcpy(oldFeature, feature[nID].szSel);

            if (nSel == CB_ERR)
            {
              //clicked outside combo - display the still-selected string
              SendMessage(hCombo, CB_SELECTSTRING, nSel, (LPARAM)NULL); //nice trick, eh?
            }
            else if (nSel == 0)
            {
              //user chose "follows" - deselect
              feature[GetFeatureIndex(oldFeature)].bUsed = FALSE; //old feature now unused
              strcpy(feature[nID].szSel, NOSEL);      //new selection
            }
            else
            {
              //save selection
              if (strcmp(feature[nID].szSel, NOSEL))
                feature[GetFeatureIndex(oldFeature)].bUsed = FALSE;  //old feature now unused
              feature[GetFeatureIndex(newFeature)].bUsed = TRUE;     //new feature used
              strcpy(feature[nID].szSel, newFeature);    //new selection
            }
          }
        }
        break;
      }
    }
  }
  return FALSE;
}


/*** AddConditionDialogProc ***/
BOOL CALLBACK AddConditionDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg)
  {
    case WM_INITDIALOG:
      SetFocus(GetDlgItem(hwnd, IDC_ADD_CONDITION));
      return TRUE;

    case WM_SYSCOMMAND:
      if (wParam ==SC_CLOSE)
      EndDialog(hwnd, 0);
      break;

    case WM_COMMAND:
    {
      switch HIWORD(wParam)
      {
        case BN_CLICKED:
        {
          switch LOWORD(wParam)
          {
            case IDOK:
            {
              LVITEM item;
              HWND parent = GetParent(hwnd);
              HWND lv = GetDlgItem(parent, IDC_CONDITION_LIST);
              int iItem = ListView_GetItemCount(lv);
              char condition[256] = "";
              char bitmap[256] = "";
              char replace[256] = "";

              GetDlgItemText(hwnd, IDC_ADD_CONDITION, condition, 256);
              GetDlgItemText(hwnd, IDC_ADD_BITMAP, bitmap, 256);
              GetDlgItemText(hwnd, IDC_ADD_REPLACE, replace, 256);

              if (!strlen(condition))
              {
                MessageBox(hwnd, "You must supply a condition.", szAppName, MB | MB_ICONEXCLAMATION);
                break;
              }

              if (!strlen(bitmap) && !strlen(replace))
              {
                MessageBox(hwnd, "You must supply a bitmap or replacement text.", szAppName, MB | MB_ICONEXCLAMATION);
                break;
              }

              item.mask = LVIF_TEXT;

              item.iItem = iItem;
              item.iSubItem = 0;
              item.pszText = condition;
              ListView_InsertItem(lv, &item);

              item.iItem = iItem;
              item.iSubItem = 1;
              item.pszText = bitmap;
              ListView_SetItem(lv, &item);

              item.iItem = iItem;
              item.iSubItem = 2;
              item.pszText = replace;
              ListView_SetItem(lv, &item);

              ListView_RedrawItems(lv, 0, iItem);
              UpdateWindow(hwnd);
              WritePrivateProfileString("Conditions", condition, bitmap, thm);
              sprintf(bitmap, "%s Text Replacement", condition);
              WritePrivateProfileString("Conditions", bitmap, replace, thm);
            }

            case IDCANCEL:
              EndDialog(hwnd, 0);
              break;
          }
        }
      }
    }
  }
  return FALSE;
}


/*** AddFavoriteDialogProc ***/
BOOL CALLBACK AddFavoriteDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg)
  {
    case WM_INITDIALOG:
    {
      HWND parent = GetParent(hwnd);
      char temp[100] = "";

      GetDlgItemText(parent, IDC_ZIPCODE, temp, 100);
      SetDlgItemText(hwnd, IDC_FAV_ZIP, temp);

      GetDlgItemText(parent, IDC_CITY, temp, 100);
      SetDlgItemText(hwnd, IDC_FAV_CITY, temp);

      GetDlgItemText(parent, IDC_STATE, temp, 100);
      SetupStateBox(hwnd, IDC_FAV_STATE, temp);

      GetDlgItemText(parent, IDC_GLOBAL_STATION, temp, 100);
      SetDlgItemText(hwnd, IDC_FAV_STATION, temp);

      SetFocus(GetDlgItem(hwnd, IDC_FAV_NAME));
      return TRUE;
    }

    case WM_SYSCOMMAND:
      if (wParam == SC_CLOSE)
      EndDialog(hwnd, 0);
      break;

    case WM_COMMAND:
    {
      switch HIWORD(wParam)
      {
        case BN_CLICKED:
        {
          switch LOWORD(wParam)
          {
            case IDOK:
            {
              char temp[256] = "";
              char name[50] = "";
              char zip[15] = "";
              char city[50] = "";
              char state[5] = "";
              char station[15] = "";

              if (nLastFav<MAX_FAVS)
              {
                GetDlgItemText(hwnd, IDC_FAV_NAME, name, 50);
                GetDlgItemText(hwnd, IDC_FAV_ZIP, zip, 15);
                GetDlgItemText(hwnd, IDC_FAV_CITY, city, 50);
                GetDlgItemText(hwnd, IDC_FAV_STATE, state, 5);
                GetDlgItemText(hwnd, IDC_FAV_STATION, station, 15);

                sprintf(temp, "Favorite%d", nLastFav);
                if (strlen(name))
                {
                  WritePrivateProfileString(szAppName, temp, name, ini);
                  AppendMenu(FavPopup, MF_ENABLED | MF_STRING, 200+nLastFav, name);
                }
                else
                {
                  MessageBox(hMainWnd, "Please specify a name that will appear in the popup menu.", szAppName, MB);
                  break;
                }

                sprintf(temp, "FavoriteZip%d", nLastFav);
                WritePrivateProfileString(szAppName, temp, zip, ini);

                sprintf(temp, "FavoriteCity%d", nLastFav);
                WritePrivateProfileString(szAppName, temp, city, ini);

                sprintf(temp, "FavoriteState%d", nLastFav);
                WritePrivateProfileString(szAppName, temp, state, ini);

                sprintf(temp, "FavoriteStation%d", nLastFav);
                WritePrivateProfileString(szAppName, temp, station, ini);

                nLastFav++;
              }
              else
                MessageBox(hMainWnd, "You can only have ten favorite cities. If this is a problem, e-mail mrjukes@purdue.edu and let me know how many you would like.", szAppName, MB);
            }
            case IDCANCEL:
              EndDialog(hwnd, 0);
              break;
          }
        }
      }
    }
  }
  return FALSE;
}


/*** ReportDialogProc ***/
BOOL CALLBACK ReportDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg)
  {
    case WM_INITDIALOG:
    {
      char buffer[1024] = "";
      char location[256] = "";
      char temp[256] = "";
      time_t now;

      switch (eUseMethod)
      {
        case useZip:
          sprintf(location, "zip code %s", loc.zip);
          break;
        case useCityState:
          sprintf(location, "%s, %s", loc.city, loc.state);
          break;
        case useStation:
          sprintf(location, "global station %s", loc.station);
          break;
      }

      time(&now);
      strcpy(temp, ctime(&now));
      temp[strlen(temp)-1] = '\0';  //trim trailing whatever-it-is
      sprintf(buffer, "Conditions for %s\r\n%s\r\n\r\n", location, temp);
      strcat(buffer, ReportCondition("Conditions"));  strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("Temperature")); strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("Humidity"));    strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("HeatIndex"));   strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("Wind"));        strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("Windchill"));   strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("Pressure"));    strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("Visibility"));  strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("Dewpoint"));    strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("Sunrise"));     strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("Sunset"));      strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("Moonrise"));    strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("Moonset"));     strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("MinTemp"));     strcat(buffer, "\r\n");
      strcat(buffer, ReportCondition("MaxTemp"));

      SetDlgItemText(hwnd, IDC_REPORT, buffer);
    }
    return TRUE;

    case WM_SYSCOMMAND:
      if (wParam == SC_CLOSE)
        EndDialog(hwnd, 0);
      break;

    case WM_COMMAND:
      if (HIWORD(wParam) == BN_CLICKED && LOWORD(wParam) == IDOK)
      EndDialog(hwnd, 0);
      break;

    case WM_KEYUP:
    case WM_KEYDOWN:
    case WM_SYSKEYUP:
    case WM_SYSKEYDOWN:
    {
      char key[4] = "";
      sprintf(key, "%d\r\nEscape = %d\r\nWM_ = %d", (int)wParam, VK_ESCAPE, msg);
      MessageBox(NULL, key, "Key pressed", IDOK);
    }
    if ((int)wParam == VK_ESCAPE)
      EndDialog(hwnd, 0);
      break;
  }
  return FALSE;
}


/*** HelpDialogProc ***/
BOOL CALLBACK HelpDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch (msg)
  {
    case WM_INITDIALOG:
    {
      char help[512] = "";
      switch (nCurrentHelp)
      {
        case IDC_THEME_FILE:    sprintf(help, "This is the name of the theme file."); break;
        case IDC_THEME_BROWSE:  sprintf(help, "Click this button to browse for a new theme file."); break;
        case IDC_TIMER:         sprintf(help, "This is the amount of time that LSWeather will wait in between checking the weather."); break;
        case IDC_DISPLAY_TEMPERATURE:
        case IDC_DISPLAY_PRESSURE:
        case IDC_DISPLAY_HUMIDITY:
        case IDC_DISPLAY_HEATINDEX:
        case IDC_DISPLAY_DEWPOINT:
        case IDC_DISPLAY_VISIBILITY:
        case IDC_DISPLAY_WIND:
        case IDC_DISPLAY_WINDCHILL:
        case IDC_DISPLAY_SUNRISE:
        case IDC_DISPLAY_SUNSET:
        case IDC_DISPLAY_MOONRISE:
        case IDC_DISPLAY_MOONSET:
        case IDC_DISPLAY_MINTEMP:
        case IDC_DISPLAY_MAXTEMP:
        case IDC_CONDITIONS_SHOW_TEXT:
        case IDC_CONDITIONS_SHOW_BMP:
        case IDC_USE_ZIPCODE:
        case IDC_USE_CITY_STATE:
        case IDC_USE_GLOBAL_STATION: sprintf(help, "Depressed = Visibile\r\nUnpressed = Hidden"); break;

        case IDC_TEMPERATURE_FORMAT: sprintf(help, "Valid Variables: %%T[fck] %%d\r\n\r\nExamples:\r\nTemp: %%Tf%%dF\r\nT: %%Tc%%dC\r\n%%Tk%%dK\r\nTemp: %%T%%dF"); break;
        case IDC_HEATINDEX_FORMAT:   sprintf(help, "Valid Variables: %%I[fck] %%d\r\n\r\nExamples:\r\nHeat Index: %%If%%dF\r\nHI: %%Ic%%dC\r\n%%Ik%%dK\r\nIndex: %%I%%dF"); break;
        case IDC_PRESSURE_FORMAT:    sprintf(help, "Valid Variables: %%P[ih]\r\n\r\nExamples:\r\nPres: %%Pi inches\r\nPres: %%Ph hPa\r\nP: %%P in."); break;
        case IDC_HUMIDITY_FORMAT:    sprintf(help, "Valid Variables: %%H\r\n\r\nExamples:\r\nHum: %%H%\r\nH: %%H %"); break;
        case IDC_DEWPOINT_FORMAT:    sprintf(help, "Valid Variables: %%D[fck] %%d\r\n\r\nExamples:\r\nDew: %%Tf%%dF\r\nDew: %%Tc%%dC\r\nDew: %%Tk%%dK\r\nDew: %%T%%dF"); break;
        case IDC_VISIBILITY_FORMAT:  sprintf(help, "Valid Variables: %%V[mk]\r\n\r\nExamples:\r\nVis: %%Vm miles\r\nVis: %%Vk km\r\nVis: %%V miles"); break;
        case IDC_WIND_FORMAT:        sprintf(help, "Valid Variables: %%W[mkd]\r\n\r\nExamples:\r\n%%Wd at %%Wmmph\r\nWind: %%Wk kph\r\nWind Direction: %%Wd"); break;
        case IDC_WINDCHILL_FORMAT:   sprintf(help, "Valid Variables: %%L[fck]\r\n\r\nExamples:\r\nWindchill Factor is %%Lf%dF\r\nWindchill: %%LkK\r\n"); break;
        case IDC_SUNRISE_FORMAT:     sprintf(help, "Valid Variables: %%SR[hmwz]\r\n\r\nExamples:\r\n%%SRh:%%SRm %%SRw (%%SRz)\r\n%%SRh:%%SRm%%SRw"); break;
        case IDC_SUNSET_FORMAT:      sprintf(help, "Valid Variables: %%SS[hmwz]\r\n\r\nExamples:\r\n%%SSh:%%SSm %%SSw (%%SSz)\r\n%%SSh:%%SSm%%SSw"); break;
        case IDC_MOONRISE_FORMAT:    sprintf(help, "Valid Variables: %%MR[hmwz]\r\n\r\nExamples:\r\n%%MRh:%%MRm %%MRw (%%MRz)\r\n%%MRh:%%MRm%%MRw"); break;
        case IDC_MOONSET_FORMAT:     sprintf(help, "Valid Variables: %%MS[hmwz]\r\n\r\nExamples:\r\n%%MSh:%%MSm %%MSw (%%MSz)\r\n%%MSh:%%MSm%%MSw"); break;
        case IDC_MINTEMP_FORMAT:     sprintf(help, "Valid Variables: %%XL[fckhmw] %%d\r\n\r\nExamples:\r\nMinimum Temp: %%XLf%%dF at %%XLh:%%XLm %%XLw\r\nMintemp: %%XLc%%dC @ %%XLh:%%XHh\r\n%%XLk%%dK\r\nmin: %%XL%%dF"); break;
        case IDC_MAXTEMP_FORMAT:     sprintf(help, "Valid Variables: %%XH[fckhmw] %%d\r\n\r\nExamples:\r\nMaximum Temp: %%XHf%%dF at %%XHh:%%XHm %%XLw\r\nMaxtemp: %%XHc%%dC @ %%XLh:%%XHh\r\n%%XHk%%dK\r\nmax: %%XH%%dF"); break;
        case IDC_CONDITIONS_FORMAT:  sprintf(help, "Valid Variables: %%C\r\n\r\nExamples:\r\n%%C\r\nCond: %%C"); break;

        case IDC_TEMPERATURE_X:
        case IDC_HEATINDEX_X:
        case IDC_PRESSURE_X:
        case IDC_HUMIDITY_X:
        case IDC_DEWPOINT_X:
        case IDC_VISIBILITY_X:
        case IDC_WIND_X:
        case IDC_WINDCHILL_X:
        case IDC_SUNRISE_X:
        case IDC_SUNSET_X:
        case IDC_MOONRISE_X:
        case IDC_MOONSET_X:
        case IDC_MINTEMP_X:
        case IDC_MAXTEMP_X:
        case IDC_CONDITIONS_TEXT_X:
        case IDC_CONDITIONS_BMP_X: sprintf(help, "The X-Coordinate where LSWeather will put the upper left corner of the corresponding value."); break;

        case IDC_TEMPERATURE_Y:
        case IDC_HEATINDEX_Y:
        case IDC_PRESSURE_Y:
        case IDC_HUMIDITY_Y:
        case IDC_DEWPOINT_Y:
        case IDC_VISIBILITY_Y:
        case IDC_WIND_Y:
        case IDC_WINDCHILL_Y:
        case IDC_SUNRISE_Y:
        case IDC_SUNSET_Y:
        case IDC_MOONRISE_Y:
        case IDC_MOONSET_Y:
        case IDC_MINTEMP_Y:
        case IDC_MAXTEMP_Y:
        case IDC_CONDITIONS_TEXT_Y:
        case IDC_CONDITIONS_BMP_Y: sprintf(help, "The Y-Coordinate where LSWeather will put the upper left corner of the corresponding value."); break;

        case IDC_FONT:            sprintf(help, "The font face name that you would like LSWeather to use.\r\n\r\nExamples:\r\nArial, System, Tahoma, Times New Roman, etc..."); break;
        case IDC_FONT_SIZE:       sprintf(help, "The size of the font that you would like LSWeather to use.\r\n\r\nExamples:\r\n8,10,12,14,16, etc..."); break;
        case IDC_FONT_NORMAL_R:   sprintf(help, "The red value of the font during normal LSWeather operation.\r\nValid Range: 0-255"); break;
        case IDC_FONT_NORMAL_G:   sprintf(help, "The green value of the font during normal LSWeather operation.\r\nValid Range: 0-255"); break;
        case IDC_FONT_NORMAL_B:   sprintf(help, "The blue value of the font during normal LSWeather operation.\r\nValid Range: 0-255"); break;
        case IDC_FONT_CHECKING_R: sprintf(help, "The red value of the font while LSWeather is updating its current conditions.\r\nValid Range: 0-255"); break;
        case IDC_FONT_CHECKING_G: sprintf(help, "The green value of the font while LSWeather is updating its current conditions.\r\nValid Range: 0-255"); break;
        case IDC_FONT_CHECKING_B: sprintf(help, "The blue value of the font while LSWeather is updating its current conditions.\r\nValid Range: 0-255"); break;

        case IDC_CONDITIONS_BMP_WIDTH: sprintf(help, "The width of each bitmap used for displaying weather conditions."); break;
        case IDC_CONDITIONS_BMP_HEIGHT: sprintf(help, "The height of each bitmap used for displaying weather conditions."); break;

        case IDC_IMAGE_DIR: sprintf(help, "The directory where all the LSWeather images are located."); break;
        case IDC_BG_IMAGE:  sprintf(help, "The bitmap that is to be displayed in the background."); break;

        case IDC_CONDITION_LIST:  sprintf(help, "This is a list of all the possible weather conditions that LSWeather will display bitmap images for.  The left column is how the weather condition would show up on wunderground.com. The middle column is the bitmap to use when the condition is found. The right column is used to replace the condition text displayed."); break;
        case IDC_BITMAP_NEW:      sprintf(help, "Click this button to add a new weather condition to the list."); break;
        case IDC_BITMAP_REMOVE:   sprintf(help, "Click this button to remove the currently selected weather condition from the list."); break;

        case IDC_ZIPCODE: sprintf(help, "Specify your zip code here."); break;
        case IDC_CITY:    sprintf(help, "Specify your city name here."); break;
        case IDC_STATE:   sprintf(help, "Pick which state you would like to use here."); break;
        case IDC_GLOBAL_STATION:    sprintf(help, "Specify your global station here. This is mostly for people outside of the USA.\r\nTo find your global station, you will have to visit http://www.wunderground.com"); break;
        case IDC_ADD_TO_FAVORITES:  sprintf(help, "Click this button to add an entry to your Favorites Menu. For now the only way to delete a favorite item is to edit your modules.ini with notepad. You can only have t favorites."); break;

        case IDC_WINDOW_WIDTH:  sprintf(help, "Specify the width of the window that you would like LSWeather to use.  This is for LoadModule only."); break;
        case IDC_WINDOW_HEIGHT: sprintf(help, "Specify the height of the window that you would like LSWeather to use.  This is for LoadModule only."); break;

        case IDC_DOUBLE_CLICK: sprintf(help, "Here is where you choose what the default double-click action is. It is pretty straight foward I believe."); break;

        case IDC_LIST_TEMP:
        case IDC_LIST_HUM:
        case IDC_LIST_DEW:
        case IDC_LIST_HI:
        case IDC_LIST_WIND:
        case IDC_LIST_WC:
        case IDC_LIST_VIS:
        case IDC_LIST_PRES:
        case IDC_LIST_MIN:
        case IDC_LIST_MAX:
        case IDC_LIST_SR:
        case IDC_LIST_SS:
        case IDC_LIST_MR:
        case IDC_LIST_MS: sprintf(help, "Choose the feature which you would like this feature to follow on the screen.  Leave the control at \"follows\" to make it the first in the list.  However, only enable one feature with \"follows\" set."); break;

        default: sprintf(help, "Help is not available for this control."); break;
      }
      SetDlgItemText(hwnd, IDC_HELPME, help);
      return TRUE;
    }
    return TRUE;

    case WM_SYSCOMMAND:
      if (wParam == SC_CLOSE)
        EndDialog(hwnd, 0);
      break;

    case WM_COMMAND:
      if (HIWORD(wParam) == BN_CLICKED && LOWORD(wParam) == IDOK)
        EndDialog(hwnd, 0);
      break;
  }
  return FALSE;
}


/*** AboutDialogProc ***/
BOOL CALLBACK AboutDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static BOOL bOverURL;
  static HBRUSH hBackground;

  switch (msg)
  {
    case WM_INITDIALOG:
      hBackground = GetSysColorBrush(COLOR_BTNFACE);
      SendMessage(GetDlgItem(hwnd, IDC_URL),        WM_SETFONT, (WPARAM)CreateFont(14, 0, 0, 0, FW_SEMIBOLD, FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, DEFAULT_PITCH, "Tahoma"), TRUE);
      SendMessage(GetDlgItem(hwnd, IDC_LSWTITLE),   WM_SETFONT, (WPARAM)CreateFont(18, 0, 0, 0, FW_BOLD,     FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, DEFAULT_PITCH, "Tahoma"), TRUE);
      SendMessage(GetDlgItem(hwnd, IDC_LSWVERSION), WM_SETFONT, (WPARAM)CreateFont(13, 0, 0, 0, FW_BOLD,     FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, DEFAULT_PITCH, "Tahoma"), TRUE);
      return TRUE;

    case WM_CTLCOLORSTATIC:
      if (GetDlgItem(hwnd, IDC_URL) == (HWND)lParam)
      {
        SetTextColor((HDC)wParam, RGB(0, 0, 255));
        SetBkMode((HDC)wParam, TRANSPARENT);
        return (BOOL)hBackground;
      }
      break;

    case WM_MOUSEMOVE:  //highlight URL 
      {
        //use the bOverURL flag to cut out flicker
        POINT  p;
        HWND   hURL;
        p.x = ((int)(short)LOWORD(lParam));
        p.y = ((int)(short)HIWORD(lParam));
        hURL = GetDlgItem(hwnd, IDC_URL);
        if (hURL == ChildWindowFromPoint(hwnd, p))
        {
          if (bOverURL) return 0;
          bOverURL = TRUE;
          SendMessage(hURL, WM_SETFONT, (WPARAM)CreateFont(14, 0, 0, 0, FW_SEMIBOLD, FALSE, TRUE, FALSE, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, DEFAULT_PITCH, "Tahoma"), TRUE);
        }
        else if (bOverURL)
        {
          bOverURL = FALSE;
          SendMessage(hURL, WM_SETFONT, (WPARAM)CreateFont(14, 0, 0, 0, FW_SEMIBOLD, FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, DEFAULT_PITCH, "Tahoma"), TRUE);
        }
      }
      return 0;

    case WM_LBUTTONDOWN:  //respond to clicks on URL
      {
        POINT  p;
        HWND   hURL;
        p.x = ((int)(short)LOWORD(lParam));
        p.y = ((int)(short)HIWORD(lParam));
         hURL  = GetDlgItem(hwnd, IDC_URL);
        if (hURL == ChildWindowFromPoint(hwnd, p))
          ShellExecute(hwnd, "open", "http://www.mccarus.com/", NULL, NULL, SW_SHOW);
      }
      return 0;

    case WM_COMMAND:
      if (wParam == IDOK || wParam == IDCANCEL)
        return EndDialog(hwnd, 0);
      break;

    case WM_SYSCOMMAND:
      if (wParam == SC_CLOSE)
        return EndDialog(hwnd, 0);
      break;
  }
  return 0;
}


/*** SetActiveCity ***/
void SetActiveCity(int x)
{
  int y=0;
  char temp[25] = "";

  sprintf(temp, "FavoriteZip%d", x);
  GetPrivateProfileString(szAppName, temp, "", loc.zip, 6, ini);
  sprintf(temp, "FavoriteCity%d", x);
  GetPrivateProfileString(szAppName, temp, "", loc.city, 50, ini);
  sprintf(temp, "FavoriteState%d", x);
  GetPrivateProfileString(szAppName, temp, "", loc.state, 5, ini);
  sprintf(temp, "FavoriteStation%d", x);
  GetPrivateProfileString(szAppName, temp, "", loc.station, 15, ini);

  WritePrivateProfileString(szAppName, "ZipCode", loc.zip, ini);
  WritePrivateProfileString(szAppName, "City", loc.city, ini);
  WritePrivateProfileString(szAppName, "State", loc.state, ini);
  WritePrivateProfileString(szAppName, "GlobalStation", loc.station, ini);

  for (y=0; y<MAX_FAVS; y++)
  {
    (y == x) ? CheckMenuItem(FavPopup, 200 + x, MF_CHECKED) : CheckMenuItem(FavPopup, 200 + y, MF_UNCHECKED);
  }

  thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)GetWeather, (LPVOID)NULL, 0, &threadID);
}


/*** ClearAllValues ***/
void ClearAllValues()
{
  __try
  {
    memset(&current.conditions, 0, sizeof(current.conditions));
    memset(&current.wind, 0, sizeof(current.wind));
    memset(&current.sunrise, 0, sizeof(current.sunrise));
    memset(&current.sunset, 0, sizeof(current.sunset));
    memset(&current.moonrise, 0, sizeof(current.moonrise));
    memset(&current.moonset, 0, sizeof(current.moonset));
    current.temp  = 0;
    current.humidity  = 0;
    current.heatindex = 0;
    current.windchill = 0;
    current.dewpoint  = 0;
    current.pressure  = 0;
    current.visibility= 0;
  }__except(1) { bErr=TRUE; }
}


/*** SetupBitmapListView ***/
void SetupBitmapListView(HWND hwnd)
{
  LVCOLUMN col;
  char buffer[8192] = "";
  char thmfile[MAX_PATH] = "";
  HWND lv = GetDlgItem(hwnd, IDC_CONDITION_LIST);

  GetDlgItemText(hwnd, IDC_THEME_FILE, thmfile, MAX_PATH);
  if (!lv) return;

  ListView_DeleteAllItems(lv);

  if (!bLoaded)
  {
    col.cx = 90;
    col.pszText = "Conditions";
    col.iSubItem = 0;
    col.mask = LVCF_SUBITEM | LVCF_TEXT | LVCF_WIDTH;
    ListView_InsertColumn(lv, 0, &col);

    col.cx = 90;
    col.pszText = "Images";
    col.iSubItem = 1;
    ListView_InsertColumn(lv, 1, &col);

    col.cx = 90;
    col.pszText = "Replace Text";
    col.iSubItem = 2;
    ListView_InsertColumn(lv, 2, &col);
  }

  GetPrivateProfileSection("Conditions", buffer, 8191, thmfile);

  if (buffer[0])
  {
    char* p = buffer;

    while (p[0] != '\0')
    {
      if (_strnicmp(p, "DisplayText=", 12) &&
          _strnicmp(p, "Format=", 7) &&
          _strnicmp(p, "TextX=", 6) &&
          _strnicmp(p, "TextY=", 6) &&
          _strnicmp(p, "DisplayBmp=", 11) &&
          _strnicmp(p, "BmpX=", 5) &&
          _strnicmp(p, "BmpY=", 5) &&
          _strnicmp(p, "BmpWidth=", 9) &&
          _strnicmp(p, "BmpHeight=", 10) &&
          _strnicmp(p, "Follows=", 8) &&
          _strnicmp(p, "Default=", 8) &&
            !strstr(p, "Text Replacement"))
      {
        char temp[256] = "";
        char temp2[256] = "";
        LVITEM item;
        int iItem = ListView_GetItemCount(lv);

        strcpy(temp, p);
        item.mask = LVIF_TEXT;

        item.iItem = iItem;
        item.iSubItem = 0;
        item.pszText = strtok(temp, "=");
        ListView_InsertItem(lv, &item);

        strcpy(temp2, temp);

        item.iItem = iItem;
        item.iSubItem = 1;
        item.pszText = strtok(NULL, "");
        ListView_SetItem(lv, &item);

        sprintf(temp, "%s Text Replacement", temp);
        GetPrivateProfileString("Conditions", temp, "", temp2, 256, thmfile);

        item.iItem = iItem;
        item.iSubItem = 2;
        item.pszText = temp2;
        ListView_SetItem(lv, &item);
      }

      p += strlen(p) + 1;
    }
  }
}


/*** SetConfigDlgValues ***/
void SetConfigDlgValues(HWND hwnd)
{
  char thmfile[MAX_PATH] = "";
  char temp[256] = "";
  int num=0;

  GetDlgItemText(hwnd, IDC_THEME_FILE, thmfile, MAX_PATH);
  SetDlgItemInt(hwnd, IDC_TIMER, GetPrivateProfileInt(szAppName, "Timer", 1, ini), FALSE);

  // Display
  SetDlgConfigDisplayValue(hwnd, "Temperature", IDC_DISPLAY_TEMPERATURE, IDC_TEMPERATURE_FORMAT, IDC_TEMPERATURE_X, IDC_TEMPERATURE_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "HeatIndex", IDC_DISPLAY_HEATINDEX, IDC_HEATINDEX_FORMAT, IDC_HEATINDEX_X, IDC_HEATINDEX_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "Pressure", IDC_DISPLAY_PRESSURE, IDC_PRESSURE_FORMAT, IDC_PRESSURE_X, IDC_PRESSURE_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "Humidity", IDC_DISPLAY_HUMIDITY, IDC_HUMIDITY_FORMAT, IDC_HUMIDITY_X, IDC_HUMIDITY_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "Dewpoint", IDC_DISPLAY_DEWPOINT, IDC_DEWPOINT_FORMAT, IDC_DEWPOINT_X, IDC_DEWPOINT_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "Visibility", IDC_DISPLAY_VISIBILITY, IDC_VISIBILITY_FORMAT, IDC_VISIBILITY_X, IDC_VISIBILITY_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "Wind", IDC_DISPLAY_WIND, IDC_WIND_FORMAT, IDC_WIND_X, IDC_WIND_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "Windchill", IDC_DISPLAY_WINDCHILL, IDC_WINDCHILL_FORMAT, IDC_WINDCHILL_X, IDC_WINDCHILL_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "Sunrise", IDC_DISPLAY_SUNRISE, IDC_SUNRISE_FORMAT, IDC_SUNRISE_X, IDC_SUNRISE_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "Sunset", IDC_DISPLAY_SUNSET, IDC_SUNSET_FORMAT, IDC_SUNSET_X, IDC_SUNSET_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "Moonrise", IDC_DISPLAY_MOONRISE, IDC_MOONRISE_FORMAT, IDC_MOONRISE_X, IDC_MOONRISE_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "Moonset", IDC_DISPLAY_MOONSET, IDC_MOONSET_FORMAT, IDC_MOONSET_X, IDC_MOONSET_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "MinTemp", IDC_DISPLAY_MINTEMP, IDC_MINTEMP_FORMAT, IDC_MINTEMP_X, IDC_MINTEMP_Y, thmfile);
  SetDlgConfigDisplayValue(hwnd, "MaxTemp", IDC_DISPLAY_MAXTEMP, IDC_MAXTEMP_FORMAT, IDC_MAXTEMP_X, IDC_MAXTEMP_Y, thmfile);

  // Font
  GetPrivateProfileString("Main", "Font", "System", temp, 256, thmfile);
  SetDlgItemText(hwnd, IDC_FONT, temp);
  SetDlgItemInt(hwnd, IDC_FONT_SIZE, GetPrivateProfileInt("Main", "FontSize", 12, thmfile), FALSE);
  GetPrivateProfileString("Main", "FontRGB", "255 255 255", temp, 256, thmfile);
  SetDlgItemInt(hwnd, IDC_FONT_NORMAL_R, atoi(strtok(temp, " ")), FALSE);
  SetDlgItemInt(hwnd, IDC_FONT_NORMAL_G, atoi(strtok(NULL, " ")), FALSE);
  SetDlgItemInt(hwnd, IDC_FONT_NORMAL_B, atoi(strtok(NULL, " ")), FALSE);
  GetPrivateProfileString("Main", "FontCheckRGB", "255 0 0", temp, 256, thmfile);
  SetDlgItemInt(hwnd, IDC_FONT_CHECKING_R, atoi(strtok(temp, " ")), FALSE);
  SetDlgItemInt(hwnd, IDC_FONT_CHECKING_G, atoi(strtok(NULL, " ")), FALSE);
  SetDlgItemInt(hwnd, IDC_FONT_CHECKING_B, atoi(strtok(NULL, " ")), FALSE);

  // Weather Conditions
  feature[cond].bDisplay ? SendDlgItemMessage(hwnd, IDC_CONDITIONS_SHOW_TEXT, BM_SETCHECK, (WPARAM)BST_CHECKED, 0) : SendDlgItemMessage(hwnd, IDC_CONDITIONS_SHOW_TEXT, BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
  SetDlgConfigDisplayValue(hwnd, "Conditions", 0, IDC_CONDITIONS_FORMAT, IDC_CONDITIONS_TEXT_X, IDC_CONDITIONS_TEXT_Y, thmfile);
  GetPrivateProfileInt("Conditions", "DisplayBmp", 0, thmfile) ? SendDlgItemMessage(hwnd, IDC_CONDITIONS_SHOW_BMP, BM_SETCHECK, (WPARAM)BST_CHECKED, 0) : SendDlgItemMessage(hwnd, IDC_CONDITIONS_SHOW_BMP, BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
  SetDlgItemInt(hwnd, IDC_CONDITIONS_BMP_X, GetPrivateProfileInt("Conditions", "BmpX", 0, thmfile), TRUE);
  SetDlgItemInt(hwnd, IDC_CONDITIONS_BMP_Y, GetPrivateProfileInt("Conditions", "BmpY", 0, thmfile), TRUE);
  SetDlgItemInt(hwnd, IDC_CONDITIONS_BMP_WIDTH, GetPrivateProfileInt("Conditions", "BmpWidth", 0, thmfile), TRUE);
  SetDlgItemInt(hwnd, IDC_CONDITIONS_BMP_HEIGHT, GetPrivateProfileInt("Conditions", "BmpHeight", 0, thmfile), TRUE);

  // General
  GetPrivateProfileString("Main", "ImageDir", "", temp, 256, thmfile);
  SetDlgItemText(hwnd, IDC_IMAGE_DIR, temp);
  GetPrivateProfileString("Main", "BackGroundPix", "", temp, 256, thmfile);
  SetDlgItemText(hwnd, IDC_BG_IMAGE, temp);
  SetDlgItemInt(hwnd, IDC_WINDOW_WIDTH, GetPrivateProfileInt("Main", "Width", 64, thmfile), TRUE);
  SetDlgItemInt(hwnd, IDC_WINDOW_HEIGHT, GetPrivateProfileInt("Main", "Height", 64, thmfile), TRUE);
  if (!bLoaded)
  {
    SendMessage(GetDlgItem(hwnd, IDC_DOUBLE_CLICK), CB_ADDSTRING, (WPARAM)0, (LPARAM)"Launch Webpage");
    SendMessage(GetDlgItem(hwnd, IDC_DOUBLE_CLICK), CB_ADDSTRING, (WPARAM)0, (LPARAM)"Change Settings");
    SendMessage(GetDlgItem(hwnd, IDC_DOUBLE_CLICK), CB_ADDSTRING, (WPARAM)0, (LPARAM)"Report Conditions");
    SendMessage(GetDlgItem(hwnd, IDC_DOUBLE_CLICK), CB_ADDSTRING, (WPARAM)0, (LPARAM)"Update");
    SendMessage(GetDlgItem(hwnd, IDC_DOUBLE_CLICK), CB_SELECTSTRING, (WPARAM)-1, (LPARAM)(LPCSTR)szDoubleClickAction);
  }

  //set Custom Format button
  bCustomFormat? SendDlgItemMessage(hwnd, IDC_CUSTOM_FORMAT, BM_SETCHECK, (WPARAM)BST_CHECKED, 0)
               : SendDlgItemMessage(hwnd, IDC_CUSTOM_FORMAT, BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);

  //set Absolute/Relative buttons
  if (bAbsolute)
  {
    SendDlgItemMessage(hwnd, IDC_ABSOLUTE, BM_SETCHECK, (WPARAM)BST_CHECKED, 0);
    SendDlgItemMessage(hwnd, IDC_RELATIVE, BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
  }
  else
  {
    SendDlgItemMessage(hwnd, IDC_ABSOLUTE, BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
    SendDlgItemMessage(hwnd, IDC_RELATIVE, BM_SETCHECK, (WPARAM)BST_CHECKED, 0);
  }
  SetDlgItemInt(hwnd, IDC_START_X, nStartX, TRUE);
  SetDlgItemInt(hwnd, IDC_START_Y, nStartY, TRUE);

  // Location
  num = GetPrivateProfileInt(szAppName, "UseMethod", 0, ini);
  SendDlgItemMessage(hwnd, IDC_USE_ZIPCODE,        BM_SETCHECK, (WPARAM)(num==useZip       ? BST_CHECKED : BST_UNCHECKED), 0);
  SendDlgItemMessage(hwnd, IDC_USE_CITY_STATE,     BM_SETCHECK, (WPARAM)(num==useCityState ? BST_CHECKED : BST_UNCHECKED), 0);
  SendDlgItemMessage(hwnd, IDC_USE_GLOBAL_STATION, BM_SETCHECK, (WPARAM)(num==useStation   ? BST_CHECKED : BST_UNCHECKED), 0);

  SetDlgItemInt(hwnd, IDC_ZIPCODE, GetPrivateProfileInt(szAppName, "ZipCode", 0, ini), FALSE);
  SetDlgItemText(hwnd, IDC_CITY, loc.city);
  SetDlgItemText(hwnd, IDC_GLOBAL_STATION, loc.station);

  // Finish it
  SetFocus(GetDlgItem(hwnd, IDC_THEME_FILE));
  SendMessage(GetDlgItem(hwnd, IDC_THEME_FILE), EM_SETSEL, 0, -1);
}


/*** SetDlgConfigDisplayValue ***/
void SetDlgConfigDisplayValue(HWND hwnd, char* section, DWORD button, DWORD format, DWORD X, DWORD Y, char* thmfile)
{
  char temp[256] = "";

  sprintf(temp, "Display%s", section);
  GetPrivateProfileInt("Main", temp, 0, thmfile) ? SendDlgItemMessage(hwnd, button, BM_SETCHECK, (WPARAM)BST_CHECKED, 0) : SendDlgItemMessage(hwnd, button, BM_SETCHECK, (WPARAM)BST_UNCHECKED, 0);
  GetPrivateProfileString(section, "Format", "", temp, 256, thmfile);
  SetDlgItemText(hwnd, format, temp);
  SetDlgItemInt(hwnd, X, GetPrivateProfileInt(section, "TextX", 0, thmfile), TRUE);
  SetDlgItemInt(hwnd, Y, GetPrivateProfileInt(section, "TextY", 0, thmfile), TRUE);
}


/*** WritePrivateProfileInt ***/
void WritePrivateProfileInt(char* appname, char* keyname, int val, char* file)
{
  char temp[256] = "";
  sprintf(temp, "%d", val);
  WritePrivateProfileString(appname, keyname, temp, file);
}


/*** WritePrivateProfileDlgString ***/
void WritePrivateProfileDlgString(char* appname, char* keyname, HWND hwnd, DWORD item, char* file)
{
  char temp[256] = "", temp2[256] = "";
  GetDlgItemText(hwnd, item, temp, 256);
  sprintf(temp2, "\"%s\"", temp);  //pad with quotes to allow leading/trailing spaces
  WritePrivateProfileString(appname, keyname, temp2, file);
}


/*** WritePrivateProfileDlgInt ***/
int WritePrivateProfileDlgInt(char* appname, char* keyname, HWND hwnd, DWORD item, char* file)
{
  char temp[256] = "";
  int val = GetDlgItemInt(hwnd, item, NULL, TRUE);
  sprintf(temp, "%d", val);
  WritePrivateProfileString(appname, keyname, temp, file);
  return val;
}


/*** SaveDlgConfigValues ***/
void SaveDlgConfigValues(HWND hwnd)
{
  char temp[256] = "";
  char thmfile[256] = "";

  GetDlgItemText(hwnd, IDC_THEME_FILE, thm, MAX_PATH);
  WritePrivateProfileString(szAppName, "ThemeFile", thm, ini);
  nTimer = GetDlgItemInt(hwnd, IDC_TIMER, NULL, FALSE) * 60000;
  WritePrivateProfileInt(szAppName, "Timer", nTimer/60000, ini);

  // Display
  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_TEMPERATURE), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplayTemperature", (feature[tmp].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplayTemperature", (feature[tmp].bDisplay = 0), thm);
  WritePrivateProfileDlgString("Temperature", "Format", hwnd, IDC_TEMPERATURE_FORMAT, thm);
  WritePrivateProfileDlgInt("Temperature", "TextX", hwnd, IDC_TEMPERATURE_X, thm);
  WritePrivateProfileDlgInt("Temperature", "TextY", hwnd, IDC_TEMPERATURE_Y, thm);
  WritePrivateProfileDlgString("Temperature", "Follows", hwnd, IDC_LIST_TEMP, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_HEATINDEX), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplayHeatIndex", (feature[hi].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplayHeatIndex", (feature[hi].bDisplay = 0), thm);
  WritePrivateProfileDlgString("HeatIndex", "Format", hwnd, IDC_HEATINDEX_FORMAT, thm);
  WritePrivateProfileDlgInt("HeatIndex", "TextX", hwnd, IDC_HEATINDEX_X, thm);
  WritePrivateProfileDlgInt("HeatIndex", "TextY", hwnd, IDC_HEATINDEX_Y, thm);
  WritePrivateProfileDlgString("HeatIndex", "Follows", hwnd, IDC_LIST_HI, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_PRESSURE), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplayPressure", (feature[pres].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplayPressure", (feature[pres].bDisplay = 0), thm);
  WritePrivateProfileDlgString("Pressure", "Format", hwnd, IDC_PRESSURE_FORMAT, thm);
  WritePrivateProfileDlgInt("Pressure", "TextX", hwnd, IDC_PRESSURE_X, thm);
  WritePrivateProfileDlgInt("Pressure", "TextY", hwnd, IDC_PRESSURE_Y, thm);
  WritePrivateProfileDlgString("Pressure", "Follows", hwnd, IDC_LIST_PRES, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_HUMIDITY), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplayHumidity", (feature[hum].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplayHumidity", (feature[hum].bDisplay = 0), thm);
  WritePrivateProfileDlgString("Humidity", "Format", hwnd, IDC_HUMIDITY_FORMAT, thm);
  WritePrivateProfileDlgInt("Humidity", "TextX", hwnd, IDC_HUMIDITY_X, thm);
  WritePrivateProfileDlgInt("Humidity", "TextY", hwnd, IDC_HUMIDITY_Y, thm);
  WritePrivateProfileDlgString("Humidity", "Follows", hwnd, IDC_LIST_HUM, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_DEWPOINT), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplayDewpoint", (feature[dew].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplayDewpoint", (feature[dew].bDisplay = 0), thm);
  WritePrivateProfileDlgString("Dewpoint", "Format", hwnd, IDC_DEWPOINT_FORMAT, thm);
  WritePrivateProfileDlgInt("Dewpoint", "TextX", hwnd, IDC_DEWPOINT_X, thm);
  WritePrivateProfileDlgInt("Dewpoint", "TextY", hwnd, IDC_DEWPOINT_Y, thm);
  WritePrivateProfileDlgString("Dewpoint", "Follows", hwnd, IDC_LIST_DEW, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_VISIBILITY), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplayVisibility", (feature[vis].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplayVisibility", (feature[vis].bDisplay = 0), thm);
  WritePrivateProfileDlgString("Visibility", "Format", hwnd, IDC_VISIBILITY_FORMAT, thm);
  WritePrivateProfileDlgInt("Visibility", "TextX", hwnd, IDC_VISIBILITY_X, thm);
  WritePrivateProfileDlgInt("Visibility", "TextY", hwnd, IDC_VISIBILITY_Y, thm);
  WritePrivateProfileDlgString("Visibility", "Follows", hwnd, IDC_LIST_VIS, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_WIND), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplayWind", (feature[wind].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplayWind", (feature[wind].bDisplay = 0), thm);
  WritePrivateProfileDlgString("Wind", "Format", hwnd, IDC_WIND_FORMAT, thm);
  WritePrivateProfileDlgInt("Wind", "TextX", hwnd, IDC_WIND_X, thm);
  WritePrivateProfileDlgInt("Wind", "TextY", hwnd, IDC_WIND_Y, thm);
  WritePrivateProfileDlgString("Wind", "Follows", hwnd, IDC_LIST_WIND, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_WINDCHILL), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplayWindchill", (feature[wc].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplayWindchill", (feature[wc].bDisplay = 0), thm);
  WritePrivateProfileDlgString("Windchill", "Format", hwnd, IDC_WINDCHILL_FORMAT, thm);
  WritePrivateProfileDlgInt("Windchill", "TextX", hwnd, IDC_WINDCHILL_X, thm);
  WritePrivateProfileDlgInt("Windchill", "TextY", hwnd, IDC_WINDCHILL_Y, thm);
  WritePrivateProfileDlgString("Windchill", "Follows", hwnd, IDC_LIST_WC, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_SUNRISE), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplaySunrise", (feature[sr].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplaySunrise", (feature[sr].bDisplay = 0), thm);
  WritePrivateProfileDlgString("Sunrise", "Format", hwnd, IDC_SUNRISE_FORMAT, thm);
  WritePrivateProfileDlgInt("Sunrise", "TextX", hwnd, IDC_SUNRISE_X, thm);
  WritePrivateProfileDlgInt("Sunrise", "TextY", hwnd, IDC_SUNRISE_Y, thm);
  WritePrivateProfileDlgString("Sunrise", "Follows", hwnd, IDC_LIST_SR, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_SUNSET), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplaySunset", (feature[ss].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplaySunset", (feature[ss].bDisplay = 0), thm);
  WritePrivateProfileDlgString("Sunset", "Format", hwnd, IDC_SUNSET_FORMAT, thm);
  WritePrivateProfileDlgInt("Sunset", "TextX", hwnd, IDC_SUNSET_X, thm);
  WritePrivateProfileDlgInt("Sunset", "TextY", hwnd, IDC_SUNSET_Y, thm);
  WritePrivateProfileDlgString("Sunset", "Follows", hwnd, IDC_LIST_SS, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_MOONRISE), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplayMoonrise", (feature[mr].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplayMoonrise", (feature[mr].bDisplay = 0), thm);
  WritePrivateProfileDlgString("Moonrise", "Format", hwnd, IDC_MOONRISE_FORMAT, thm);
  WritePrivateProfileDlgInt("Moonrise", "TextX", hwnd, IDC_MOONRISE_X, thm);
  WritePrivateProfileDlgInt("Moonrise", "TextY", hwnd, IDC_MOONRISE_Y, thm);
  WritePrivateProfileDlgString("Moonrise", "Follows", hwnd, IDC_LIST_MR, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_MOONSET), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplayMoonset", (feature[ms].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplayMoonset", (feature[ms].bDisplay = 0), thm);
  WritePrivateProfileDlgString("Moonset", "Format", hwnd, IDC_MOONSET_FORMAT, thm);
  WritePrivateProfileDlgInt("Moonset", "TextX", hwnd, IDC_MOONSET_X, thm);
  WritePrivateProfileDlgInt("Moonset", "TextY", hwnd, IDC_MOONSET_Y, thm);
  WritePrivateProfileDlgString("Moonset", "Follows", hwnd, IDC_LIST_MS, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_MINTEMP), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplayMinTemp", (feature[min].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplayMinTemp", (feature[min].bDisplay = 0), thm);
  WritePrivateProfileDlgString("MinTemp", "Format", hwnd, IDC_MINTEMP_FORMAT, thm);
  WritePrivateProfileDlgInt("MinTemp", "TextX", hwnd, IDC_MINTEMP_X, thm);
  WritePrivateProfileDlgInt("MinTemp", "TextY", hwnd, IDC_MINTEMP_Y, thm);
  WritePrivateProfileDlgString("MinTemp", "Follows", hwnd, IDC_LIST_MIN, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_DISPLAY_MAXTEMP), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "DisplayMaxTemp", (feature[max].bDisplay = 1), thm) : WritePrivateProfileInt("Main", "DisplayMaxTemp", (feature[max].bDisplay = 0), thm);
  WritePrivateProfileDlgString("MaxTemp", "Format", hwnd, IDC_MAXTEMP_FORMAT, thm);
  WritePrivateProfileDlgInt("MaxTemp", "TextX", hwnd, IDC_MAXTEMP_X, thm);
  WritePrivateProfileDlgInt("MaxTemp", "TextY", hwnd, IDC_MAXTEMP_Y, thm);
  WritePrivateProfileDlgString("MaxTemp", "Follows", hwnd, IDC_LIST_MAX, thm);

  // Font
  GetDlgItemText(hwnd, IDC_FONT, szFontName, MAX_PATH);
  WritePrivateProfileString("Main", "Font", szFontName, thm);
  nFontSize = GetDlgItemInt(hwnd, IDC_FONT_SIZE, NULL, FALSE);
  WritePrivateProfileInt("Main", "FontSize", nFontSize, thm);
  R = GetDlgItemInt(hwnd, IDC_FONT_NORMAL_R, NULL, FALSE);
  G = GetDlgItemInt(hwnd, IDC_FONT_NORMAL_G, NULL, FALSE);
  B = GetDlgItemInt(hwnd, IDC_FONT_NORMAL_B, NULL, FALSE);
  sprintf(temp, "%d %d %d", R, G, B);
  WritePrivateProfileString("Main", "FontRGB", temp, thm);

  CR = GetDlgItemInt(hwnd, IDC_FONT_CHECKING_R, NULL, FALSE);
  CG = GetDlgItemInt(hwnd, IDC_FONT_CHECKING_G, NULL, FALSE);
  CB = GetDlgItemInt(hwnd, IDC_FONT_CHECKING_B, NULL, FALSE);
  sprintf(temp, "%d %d %d", CR, CG, CB);
  WritePrivateProfileString("Main", "FontCheckRGB", temp, thm);

  // Weather Conditions
  (SendMessage(GetDlgItem(hwnd, IDC_CONDITIONS_SHOW_TEXT), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Conditions", "DisplayText", (feature[cond].bDisplay = 1), thm) : WritePrivateProfileInt("Conditions", "DisplayText", (feature[cond].bDisplay = 0), thm);
  WritePrivateProfileDlgString("Conditions", "Format", hwnd, IDC_CONDITIONS_FORMAT, thm);
  WritePrivateProfileDlgInt("Conditions", "TextX", hwnd, IDC_CONDITIONS_TEXT_X, thm);
  WritePrivateProfileDlgInt("Conditions", "TextY", hwnd, IDC_CONDITIONS_TEXT_Y, thm);
  WritePrivateProfileDlgString("Conditions", "Follows", hwnd, IDC_LIST_COND, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_CONDITIONS_SHOW_BMP), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Conditions", "DisplayBmp", 1, thm) : WritePrivateProfileInt("Conditions", "DisplayBmp", 0, thm);
  WritePrivateProfileDlgInt("Conditions", "BmpX", hwnd, IDC_CONDITIONS_BMP_X, thm);
  WritePrivateProfileDlgInt("Conditions", "BmpY", hwnd, IDC_CONDITIONS_BMP_Y, thm);
  WritePrivateProfileDlgInt("Conditions", "BmpWidth", hwnd, IDC_CONDITIONS_BMP_WIDTH, thm);
  WritePrivateProfileDlgInt("Conditions", "BmpHeight", hwnd, IDC_CONDITIONS_BMP_HEIGHT, thm);

  // General
  GetDlgItemText(hwnd, IDC_IMAGE_DIR, szImageDir, MAX_PATH);
  if (szImageDir[strlen(szImageDir)-1] != '\\') strcat(szImageDir, "\\");
  WritePrivateProfileString("Main", "ImageDir", szImageDir, thm);

  if (bLoadModule)
  {
    WndWidth = WritePrivateProfileDlgInt("Main", "Width", hwnd, IDC_WINDOW_WIDTH, thm);
    WndHeight = WritePrivateProfileDlgInt("Main", "Height", hwnd, IDC_WINDOW_HEIGHT, thm);
  }
  else
  {
    WndWidth = WndHeight = wndSize;
  }

  GetDlgItemText(hwnd, IDC_BG_IMAGE, temp, 256);
  WritePrivateProfileString("Main", "BackGroundPix", temp, thm);
  if (temp[0])
  {
    char bmp[256] = "";
    sprintf(bmp, "%s%s", szImageDir, temp);
    if (BGBMP) DeleteObject(BGBMP);
    BGBMP = LoadImage(Instance, bmp, IMAGE_BITMAP, WndWidth, WndHeight, LR_LOADFROMFILE);
  }

  SendMessage(GetDlgItem(hwnd, IDC_DOUBLE_CLICK), CB_GETLBTEXT, (WPARAM)SendMessage(GetDlgItem(hwnd, IDC_DOUBLE_CLICK), CB_GETCURSEL, 0, 0), (LPARAM)(LPCSTR)szDoubleClickAction);
  WritePrivateProfileString("Main", "DoubleClick", szDoubleClickAction, thm);

  (SendMessage(GetDlgItem(hwnd, IDC_CUSTOM_FORMAT), BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "CustomFormat", (bCustomFormat = 1), thm) : WritePrivateProfileInt("Main", "CustomFormat", (bCustomFormat = 0), thm);
  (SendMessage(GetDlgItem(hwnd, IDC_ABSOLUTE),      BM_GETCHECK, 0, 0) == BST_CHECKED)? WritePrivateProfileInt("Main", "AbsolutePlacement", (bAbsolute = 1), thm) : WritePrivateProfileInt("Main", "AbsolutePlacement", (bAbsolute = 0), thm);
  nStartX = GetDlgItemInt(hwnd, IDC_START_X, NULL, TRUE);
  nStartY = GetDlgItemInt(hwnd, IDC_START_Y, NULL, TRUE);
  WritePrivateProfileInt("Main", "StartX", nStartX, thm);
  WritePrivateProfileInt("Main", "StartY", nStartY, thm);

  // Location
  if (SendMessage(GetDlgItem(hwnd, IDC_USE_ZIPCODE), BM_GETCHECK, 0, 0) == BST_CHECKED)
    SendMessage(hMainWnd, WM_COMMAND, (WPARAM)M_USEZIP, (LPARAM)0);
  else if (SendMessage(GetDlgItem(hwnd, IDC_USE_CITY_STATE), BM_GETCHECK, 0, 0) == BST_CHECKED)
    SendMessage(hMainWnd, WM_COMMAND, (WPARAM)M_USECITY, (LPARAM)0);
  else if (SendMessage(GetDlgItem(hwnd, IDC_USE_GLOBAL_STATION), BM_GETCHECK, 0, 0) == BST_CHECKED)
    SendMessage(hMainWnd, WM_COMMAND, (WPARAM)M_USESTATION, (LPARAM)0);

  GetDlgItemText(hwnd, IDC_ZIPCODE, loc.zip, 6);
  WritePrivateProfileString(szAppName, "ZipCode", loc.zip, ini);
  GetDlgItemText(hwnd, IDC_CITY, loc.city, 50);
  WritePrivateProfileString(szAppName, "City", loc.city, ini);

  SendMessage(GetDlgItem(hwnd, IDC_STATE), CB_GETLBTEXT, (WPARAM)SendMessage(GetDlgItem(hwnd, IDC_STATE), CB_GETCURSEL, 0, 0), (LPARAM)(LPCSTR)loc.state);
  WritePrivateProfileString(szAppName, "State", loc.state, ini);
  GetDlgItemText(hwnd, IDC_GLOBAL_STATION, loc.station, 15);
  WritePrivateProfileString(szAppName, "GlobalStation", loc.station, ini);

  //save order of relative placements
  if (!bAbsolute)
    GetFeatureOrder();
}


/*** SetupStateBox ***/
void SetupStateBox(HWND hwnd, DWORD cbox, char* st)
{
  HWND cb = GetDlgItem(hwnd, cbox);

  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"AL");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"AK");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"AZ");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"AR");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"CA");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"CO");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"CT");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"DE");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"DC");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"FL");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"GA");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"HI");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"ID");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"IL");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"IN");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"IA");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"KS");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"KY");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"LA");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"ME");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"MD");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"MA");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"MI");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"MN");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"MS");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"MO");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"MT");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"NE");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"NV");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"NH");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"NJ");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"NM");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"NY");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"NC");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"ND");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"OH");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"OK");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"OR");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"PA");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"RI");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"SC");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"SD");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"TN");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"TX");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"UT");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"VT");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"VA");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"WA");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"WV");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"WI");
  SendMessage(cb, CB_ADDSTRING, (WPARAM)0, (LPARAM)"WY");

  SendMessage(cb, CB_SELECTSTRING, (WPARAM)-1, (LPARAM)(LPCSTR)st);
}


/*** ParseFormat ***/
void ParseFormat(char* in, char* out)
{
  __try
  {
    int CURRENT_VALUE = 0;
    BOOL MOON = FALSE;
    char temp[256] = "";
    int x=0, y=0;

    for (x=0; in[x]; x++)
    {
      switch (in[x])
      {
        case '%':
        {
          x++;

          switch (in[x])
          {
            case 'd': sprintf(temp, ""); break;
            case 'D': CURRENT_VALUE = current.dewpoint;
            case 't':
            case 'T':
            {
              if (toupper(in[x]) == 'T') CURRENT_VALUE = current.temp;
              x++;

              switch (toupper(in[x]))
              {
                default: x--;
                case 'F': sprintf(temp, "%d", CURRENT_VALUE); break;
                case 'C': sprintf(temp, "%d", FTOC(CURRENT_VALUE)); break;
                case 'K': sprintf(temp, "%d", FTOK(CURRENT_VALUE)); break;
              }
            }
            break;

            case 'i':
            case 'I':
            {
              x++;
              switch (toupper(in[x]))
              {
                default: x--;
                case 'F': sprintf(temp, "%d", current.heatindex); break;
                case 'C': sprintf(temp, "%d", FTOC(current.heatindex)); break;
                case 'K': sprintf(temp, "%d", FTOK(current.heatindex)); break;
              }
            }
            break;

            case 'l':
            case 'L':
            {
              x++;
              switch (toupper(in[x]))
              {
                default: x--;
                case 'F': sprintf(temp, "%d", current.windchill); break;
                case 'C': sprintf(temp, "%d", FTOC(current.windchill)); break;
                case 'K': sprintf(temp, "%d", FTOK(current.windchill)); break;
              }
            }
            break;

            case 'p':
            case 'P':
            {
              x++;
              switch (toupper(in[x]))
              {
                default: x--;
                case 'I': sprintf(temp, "%4.2f", current.pressure); break;
                case 'H': sprintf(temp, "%4.2f", ITOH(current.pressure)); break;
              }
            }
            break;

            case 'h':
            case 'H': sprintf(temp, "%d", current.humidity); break;

            case 'c':
            case 'C': sprintf(temp, "%s", current.conditions); break;

            case 'v':
            case 'V':
            {
              x++;
              switch (toupper(in[x]))
              {
                default: x--;
                case 'M': sprintf(temp, "%d", current.visibility); break;
                case 'K': sprintf(temp, "%d", MTOK(current.visibility)); break;
              }
            }
            break;

            case 'w':
            case 'W':
            {
              x++;
              strcpy(temp, current.wind);

              switch(toupper(in[x]))
              {
                default: x--;
                case 'M':
                {
                  if (_strnicmp(temp, "Calm", 4))
                  {
                    char* s = strtok(temp, " ");
                    strtok(NULL, " ");
                    s = strtok(NULL, " ");

                    if (s) sprintf(temp, "%d", atoi(s));
                    else sprintf(temp, "0");
                  }
                  else sprintf(temp, "0");
                }
                break;

                case 'K':
                {
                  if (_strnicmp(temp, "Calm", 4))
                  {
                    char* s = strtok(temp, " ");
                    strtok(NULL, " ");
                    s = strtok(NULL, " ");

                    if (s) sprintf(temp, "%d", MTOK(atoi(s)));
                    else sprintf(temp, "0");
                  }
                  else sprintf(temp, "0");
                }
                break;

                case 'D':
                {
                  char* d = strtok(temp, " ");
                  if (d)
                  {
                    switch (nWindStyle)
                    {
                      case 0: //ALL CAPS - NNE or SOUTH
                        sprintf(temp, "%s", strupr(d)); break;
                      case 1: //First Caps - NNE or South (default value)
                        sprintf(temp, "%s", d); break;
                      case 2: //ABBR. caps - NNE or south
                        sprintf(temp, "%s", strlen(d) < 4 ? d : strlwr(d)); break;
                      case 3: //no caps - nne or south
                        sprintf(temp, "%s", strlwr(d)); break;
                      default:
                        sprintf(temp, "%s", d); break; //same as case 1, I know
                    }
                  }
                  else
                    sprintf(temp, "N/A");
                }
                break;
              }
            }
            break;

            case 'm':
            case 'M': MOON=TRUE;
            case 's':
            case 'S':
            {
              if (toupper(in[x]) == 'S') MOON=FALSE;
              x++;
              switch (toupper(in[x]))
              {
                default:
                case 'R': MOON ? strcpy(temp, current.moonrise) : strcpy(temp, current.sunrise);
                case 'S':
                {
                  if (toupper(in[x]) == 'S') MOON ? strcpy(temp, current.moonset) : strcpy(temp, current.sunset);

                  x++;
                  switch (toupper(in[x]))
                  {
                    default: x--;
                    case 'H':
                    {
                      char* hour = strtok(temp, ":");
                      if (hour) sprintf(temp, "%d", atoi(hour));
                      else sprintf(temp, "0");
                    }
                    break;

                    case 'M':
                    {
                      char* min = strtok(temp, ":");
                      min = strtok(NULL, " ");
                      sprintf(temp, "%02d", atoi(min));
                    }
                    break;

                    case 'W':
                    {
                      if (strstr(temp, "AM")) sprintf(temp, "am");
                      else if (strstr(temp, "PM")) sprintf(temp, "pm");
                      else sprintf(temp, "NA");
                    }
                    break;

                    case 'Z':
                    {
                      char* zone = strtok(temp, "(");
                      zone = strtok(NULL, ")");
                      sprintf(temp, "%s", zone);
                    }
                    break;
                  }
                }
              break;
              }
            }
            break;

            case 'x':
            case 'X':
            {
              x++;
              switch (toupper(in[x]))
              {
                case 'H':
                {
                  x++;
                  switch (toupper(in[x]))
                  {
                    default: x--;
                    case 'F': sprintf(temp, "%d", tempMax.temp); break;
                    case 'C': sprintf(temp, "%d", FTOC(tempMax.temp)); break;
                    case 'K': sprintf(temp, "%d", FTOK(tempMax.temp)); break;
                    case 'H': sprintf(temp, "%d", tempMax.hour%12==0? 12 : tempMax.hour % 12); break;
                    case 'M': sprintf(temp, "%02d", tempMax.min); break;
                    case 'W': sprintf(temp, "%s", tempMax.hour>=12 ? "pm" : "am"); break;
                  }
                }
                break;

                case 'L':
                {
                  x++;
                  switch (toupper(in[x]))
                  {
                    default: x--;
                    case 'F': sprintf(temp, "%d", tempMin.temp); break;
                    case 'C': sprintf(temp, "%d", FTOC(tempMin.temp)); break;
                    case 'K': sprintf(temp, "%d", FTOK(tempMin.temp)); break;
                    case 'H': sprintf(temp, "%d", tempMin.hour%12==0 ? 12 : tempMin.hour % 12); break;
                    case 'M': sprintf(temp, "%02d", tempMin.min); break;
                    case 'W': sprintf(temp, "%s", tempMin.hour>=12 ? "pm" : "am"); break;
                  }
                }
                break;

                default: x--;
              }
            }
            break;

            default:
            {
              x--;
              sprintf(temp, "%c", in[x]);
            }
            break;
          }

          strcat(out, temp);
          y = strlen(out);
        }
        break;

        case '#':  //replace '#xxx' with ASCII character xxx
        {
          int ascii = 0, i = 0;
          x++;
          while (isdigit(in[x]) && i<3)
          {
            ascii = 10 * ascii + atoi(&in[x]);
            i++; x++;
          }
          if (i==0) //no digits following #
          {
            if (in[x]=='#')  //catch '##' string
              out[y] = '#';
            else
              x--;  //'#' alone is discarded
          }
          else
          {
            out[y] = ascii;
            x--;
          }
          y++;
        }
        break;

        default:
        {
          out[y] = in[x];
          y++;
        }
      }
    }
  }__except(1) { bErr=TRUE; }
}


/*** ConstructURL ***/
BOOL ConstructURL(char* FULL_URL)
{
  __try {
  BOOL GO=TRUE;

  strcpy(FULL_URL, g_szBaseURL);

  if (eUseMethod == useZip)
  {
    if (loc.zip[0])
    {
      strcat(FULL_URL, "US/");
      strcat(FULL_URL, loc.zip);
    }
    else GO=FALSE;
  }
  else if (eUseMethod == useCityState)
  {
    if (loc.state[0] && loc.city[0])
    {
      char temp[75] = "";
      strcpy(temp, loc.city);

      strcat(FULL_URL, "US/");
      strcat(FULL_URL, loc.state);

      while (strstr(temp, " "))
      {
        temp[strcspn(temp, " ")] = '_';
      }

      strcat(FULL_URL, "/");
      strcat(FULL_URL, temp);
    }
    else GO=FALSE;
  }
  else if (eUseMethod == useStation)
  {
    if (loc.station[0])
    {
      strcat(FULL_URL, "global/stations/");
      strcat(FULL_URL, loc.station);
    }
    else GO=FALSE;
  }
  else GO=FALSE;

  if (GO) strcat(FULL_URL, ".html");
    return TRUE;
  }__except(1) { bErr=TRUE; return FALSE; }
}


/*** ReportCondition ***/
char* ReportCondition(char* what)
{
  __try
  {
    char temp[256] = "";

    memset(szReportString, 0, sizeof(szReportString));
    GetPrivateProfileString(what, bCustomFormat ? "Format" : "Default", "", temp, 256, thm);
    ParseFormat(temp, szReportString);
    return szReportString;
  }__except(1) { bErr=TRUE; return ""; }
}


/*** Bang Commands ***/
void BangHide(HWND w, char* a) { ShowWindow(hMainWnd, SW_HIDE); bHidden=TRUE; }
void BangShow(HWND w, char* a) { ShowWindow(hMainWnd, SW_SHOW); bHidden=FALSE; }
void BangToggle(HWND w, char* a) { bHidden ? BangShow(NULL, "") : BangHide(NULL, ""); }
void BangUpdate(HWND w, char* a) { SendMessage(hMainWnd, WM_COMMAND, (WPARAM)M_UPDATE, 0); }
void BangReport(HWND w, char* a) { SendMessage(hMainWnd, WM_COMMAND, (WPARAM)M_REPORT, 0); }
void BangChangeSettings(HWND w, char* a) { SendMessage(hMainWnd, WM_COMMAND, (WPARAM)M_SETTINGS, 0); }
void BangLaunchWebpage(HWND w, char* a) { SendMessage(hMainWnd, WM_COMMAND, (WPARAM)M_LAUNCH, 0); }
void BangAlwaysOnTop(HWND w, char* a)
{
  if (bOnTop) { SetWindowPos(hMainWnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE); bOnTop=FALSE; }
         else { SetWindowPos(hMainWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE); bOnTop=TRUE; }
}


/*** DisplayValue ***/
void DisplayValue(HDC buf, char* val)
{
  __try
  {
    char temp[256] = "";
    char out[256] = "";

    //get format, checking if wind is calm...
    GetPrivateProfileString(val, "Format", "", temp, 256, thm);
    if (!strcmp(val, "Wind") && strstr(current.wind, "Calm"))
      GetPrivateProfileString(val, "Calm", temp, temp, 256, thm); //if not found use regular
    if (strlen(temp)) ParseFormat(temp, out);

    /*********************/
    /******* DEBUG *******/
    /*********************/
#ifdef _DEBUG
    if (bErr && !strcmp(val, "Conditions"))
    {
      sprintf(temp, "ERROR!! %s", out);
      strcpy(out, temp);
    }
#endif
    /*********************/

    if (bAbsolute)
    {
      TextOut(buf,
        GetPrivateProfileInt(val, "TextX", 5, thm),
        GetPrivateProfileInt(val, "TextY", 5, thm),
        out, strlen(out));
    }
    else //much more fun!
    {
      RECT r;
      int thisone, thatone;
      char preceder[20];
      GetPrivateProfileString(val, "Follows", NOSEL, preceder, 20, thm);

      if (!strcmp(preceder, NOSEL))
      {
        SetRect(&r, nStartX, nStartY, WndWidth, WndHeight);
      }
      else
      {
        thatone = GetFeatureIndex(preceder);
        SetRect(&r, feature[thatone].x, feature[thatone].y, WndWidth, WndHeight);
      }
      DrawText(buf, out, strlen(out), &r, DT_CALCRECT | DT_LEFT | DT_VCENTER);
      DrawText(buf, out, strlen(out), &r, DT_LEFT | DT_VCENTER);

      //save item dimensions
      thisone = GetFeatureIndex(val);
      feature[thisone].x = r.right;
      feature[thisone].y = r.top;
    }
  }__except(1) { bErr=TRUE; }
}


/*** BrowseForFile ***/
void BrowseForFile(HWND hwnd, DWORD editbox, char* filter)
{
  char thmfile[256] = "";
  OPENFILENAME* of  = malloc(sizeof(OPENFILENAME));

  memset(&of, 0, sizeof(of));
  of->lStructSize   = sizeof(OPENFILENAME);
  of->hwndOwner     = hwnd;
  of->lpstrFilter   = filter;
  of->lpstrFile     = thmfile;
  of->nMaxFile      = MAX_PATH;
  of->lpstrTitle    = "LSWeather Browse";
  of->Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR;

  __try
  {
    if (GetOpenFileName(of))
    {
      //load theme settings automatically
      SetDlgItemText(hwnd, editbox, thmfile);
      SetupBitmapListView(hwnd);
      SetConfigDlgValues(hwnd);
    }
  } __except(1) { }

  free(of);
}


/*** GetFeatureIndex ***/
int GetFeatureIndex(char* featureName)
{
  int i;
  for (i = 0; i < NUM_FEATURES; i++)
  {
    if (!strcmp(feature[i].szName, featureName))
      return i;
  }
  return -1;
}


/*** GetFeatureOrder ***/
void GetFeatureOrder()
{
  int i, open[NUM_FEATURES];
  char szSearchName[20] = NOSEL;

  //init variables
  strcpy(szOrder, "");
  for (i = 0; i < NUM_FEATURES; i++)
    open[i] = feature[i].bDisplay;

  //single-loop attempt
  //note: this DOES NOT support multiple boxes with "follows"

  //scan loop repeatedly, always searching for next follower
  for (i = 0; i < NUM_FEATURES; i++)
  {
    if (open[i] && !strcmp(szSearchName, feature[i].szSel))
    {
      //found its follower
      szOrder[strlen(szOrder)+1] = 0;
      szOrder[strlen(szOrder)] = i + 65;  //append to order string
      strcpy(szSearchName, feature[i].szName);
      open[i] = FALSE;
      i = -1; //always search from beginning!
    } //end if match found
  }   //end for
}


/*** ShowControls ***/
void ShowControls(HWND hwnd, BOOL bRel)
{
  //to show all relative controls only, call ShowControls(TRUE);
  //to show all absolute controls only, call ShowControls(FALSE);

  int i, both = SW_HIDE + SW_SHOWNORMAL;
  int ShowType = bRel ? SW_SHOWNORMAL : SW_HIDE;

  for (i = REL_START; i <= REL_END; i++)
    ShowWindow(GetDlgItem(hwnd, i), ShowType);

  for (i = ABS_START; i <= ABS_END; i++)
    ShowWindow(GetDlgItem(hwnd, i), both - ShowType);
}
