/*

  This is a part of the LiteStep Shell Source code.

  Copyright(C) 1997 - 98 Francis Gastellu
                    aka Lone Runner/Aegis

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or(at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111 - 1307, USA.

*/
/*********************************************************************/
/*                                                                   */
/* LiteSTEP Wharf Module SDK 1.4                                     */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* Header file                                                       */
/*                                                                   */
/* ----------------------------------------------------------------- */

#ifndef __LSSDK_H
#define __LSSDK_H

/* ----------------------------------------------------------------- */

#include <windows.h>
#include "wharfdata.h"


/*** exports ***/
#ifdef __cplusplus
extern "C"
{
#endif

  __declspec(dllexport) int initModule(HWND parent, HINSTANCE dll, wharfDataType* wd);
  __declspec(dllexport) int initModuleEx(HWND parent, HINSTANCE dll, LPCSTR szPath);
  __declspec(dllexport) void quitModule(HINSTANCE dllInst);

  __declspec(dllexport) void CALLBACK GetWeather();

  __declspec(dllexport) int initWharfModule(HWND parent, HINSTANCE dll, wharfDataType *wd);
  __declspec(dllexport) void quitWharfModule(HINSTANCE dllInst);

#ifdef __cplusplus
}
#endif

#endif


/*** Constants ***/
enum {useZip, useCityState, useStation};
enum {tmp, hum, dew, hi, wind, wc, vis, pres, min, max, sr, ss, mr, ms, cond};

#define M_UPDATE     100
#define M_ACTIVE     101
#define M_SETTINGS   102
#define M_ABOUT      103
#define M_USEZIP     104
#define M_USECITY    105
#define M_USESTATION 106
#define M_FARENHEIT  107
#define M_REPORT     108
#define M_LAUNCH     109
#define M_RESET      110

#define MAX_FAVS 10             //maximum number of favorites
#define SNAP_DIST 10            //snap window to edge if within this many pixels
#define NUM_FEATURES 15         //number of weather features
#define NOSEL "follows"
#define MB MB_OK | MB_SETFOREGROUND

#define REL_START   IDC_LIST_TEMP       //lowest "relative" control...
#define REL_END     IDC_START_Y         //highest.
#define ABS_START   IDC_TEMPERATURE_X   //lowest "absolute" control...
#define ABS_END     IDC_ABS_Y2          //highest.  See resource.h.

#define TEMP_FILE   "lsweather.tmp"     //URLDownloadToFile destination

#define ERROR_RETURN  { bErr = TRUE; return; }

/*** Prototypes ***/
void BrowseForFile(HWND hwnd, DWORD editbox, char* filter);
void ClearAllValues();
BOOL ConstructURL(char* FULL_URL);
void DisplayValue(HDC buf, char* val);
BOOL GetValue(const char* pointer, char* temp, char* delim);
void GetWind(const char* pointer, char* temp);
void LoadSetup();
void LoadThemeInfo();
void ParseFormat(char*, char*);
char* ReportCondition(char* what);
void SaveDlgConfigValues(HWND hwnd);
void SetActiveCity(int);
void SetConfigDlgValues(hwnd);
void SetDlgConfigDisplayValue(HWND hwnd, char* section, DWORD button, DWORD format, DWORD x, DWORD y, char* thmfile);
void SetupBitmapListView(HWND hwnd);
void SetWindowBitmapRgn(HWND hwnd, HBITMAP bmp);
void SetupStateBox(HWND hwnd, DWORD, char*);
void WritePrivateProfileInt(char* appname, char* keyname, int val, char* file);
int  WritePrivateProfileDlgInt(char* appname, char* keyname, HWND hwnd, DWORD item, char* file);
void WritePrivateProfileDlgString(char* appname, char* keyname, HWND hwnd, DWORD item, char* file);
void ShowControls(HWND hwnd, BOOL bRel);
void GetFeatureOrder();
int GetFeatureIndex(char* featureName);

void BangHide(HWND, char*);
void BangShow(HWND, char*);
void BangToggle(HWND, char*);
void BangUpdate(HWND, char*);
void BangReport(HWND, char*);
void BangChangeSettings(HWND, char*);
void BangLaunchWebpage(HWND, char*);
void BangAlwaysOnTop(HWND, char*);

/*** Dialog/Window Procedures ***/
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK AboutDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK AddConditionDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK AddFavoriteDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK ConfigDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK HelpDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK ReportDialogProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);


/*** Structs ***/
struct feature_t    // this struct is used for relative display
{
  BOOL bUsed;       // is this one being followed already?
  int  bDisplay;    // display this feature?
  char szName[12];  // name of feature
  char szSel[12];   // selected string in corresponding (by #) combo box
  long x, y;        // RIGHT and TOP coords of feature text on screen
} feature[NUM_FEATURES];

struct current_t
{
  float pressure;
  int temp, humidity, dewpoint;
  int visibility, heatindex, windchill;
  char conditions[50], wind[50];
  char sunrise[25], sunset[25];
  char moonrise[25], moonset[25];
} current;

struct loc_t // holds location information
{
  char city[50], state[3], zip[6], station[6];
} loc;

struct minmax_t
{
  int temp;  // actual temperature
  int day;   // day of occurence
  int hour;  // hour of occurence
  int min;   // hour of occurence
} tempMax, tempMin;
