#include <time.h>
#include <math.h>
#include <stdio.h>
#include "lsweather.h"


/*** Globals main module ***/
extern int  nTimer;
extern BOOL bChecking;
extern BOOL bErr;
extern HWND hMainWnd;
extern char ini[];


/*** GetWeather ***/
void CALLBACK GetWeather()
{
  __try
  {
    if (!bChecking && nTimer >= 60000)
    {
      char  szURL[256];
      char  data[256] = "";
      char  szTemp[65536];
      char  *sp;
      FILE  *fp;

      bChecking = TRUE;
      bErr = FALSE;
      InvalidateRect(hMainWnd, NULL, TRUE);
      UpdateWindow(hMainWnd);

      if (!ConstructURL(szURL)) ERROR_RETURN;
      if (S_OK != URLDownloadToFile(NULL, szURL, TEMP_FILE, 0, NULL)) ERROR_RETURN;
      ClearAllValues();

      //read file into buffer
      fp = fopen(TEMP_FILE, "rt");
      if (fp)
      {
        fread(szTemp, sizeof(char), sizeof(szTemp), fp);
        fclose(fp);
        DeleteFile(TEMP_FILE);
      }
      else
      {
        DeleteFile(TEMP_FILE);
        ERROR_RETURN;
      }

      if ((sp = strstr(szTemp, ">Temperature</td>")) != NULL)
      {
        if (GetValue(sp, data, "<"))
          current.temp = atoi(data);
        else ERROR_RETURN;
      }

      if ((sp = strstr(szTemp, "<td>Humidity</td>")) != NULL)
      {
        if (GetValue(sp, data, "<"))
          current.humidity = atoi(data);
        else ERROR_RETURN;
      }

      if ((sp = strstr(szTemp, "<td>Dew Point</td>")) != NULL)
      {
        if (GetValue(sp, data, "<"))
          current.dewpoint = atoi(data);
        else ERROR_RETURN;
      }

      if ((sp = strstr(szTemp, "<td>Wind</td>")) != NULL)
        GetWind(sp, current.wind);

      if ((sp = strstr(szTemp, "<td>Pressure</td>")) != NULL)
      {
        if (GetValue(sp, data, "<"))
          current.pressure = (float)atof(data);
        else ERROR_RETURN;
      }

      if ((sp = strstr(szTemp, "<td>Conditions</td>")) != NULL)
      {
        if (!GetValue(sp, current.conditions, "<")) ERROR_RETURN
      }

      if ((sp = strstr(szTemp, "<td>Visibility</td>")) != NULL)
      {
        if (GetValue(sp, data, "<"))
          current.visibility = atoi(data);
        else ERROR_RETURN;
      }

      if ((sp = strstr(szTemp, "<td>Sunrise</td>")) != NULL)
      {
        if (!GetValue(sp, current.sunrise, "<")) ERROR_RETURN;
      }

      if ((sp = strstr(szTemp, "<td>Sunset</td>")) != NULL)
      {
        if (!GetValue(sp, current.sunset, "<")) ERROR_RETURN;
      }

      if ((sp = strstr(szTemp, "<td>Moon Rise</td>")) != NULL)
      {
        sp = strstr(sp, "<b>");
        if (*(sp - 1) == '(') sp = strstr(sp + 1, "<b>");  //skip "(<b>5/31</b>)"
        if (!GetValue(sp, current.moonrise, "<")) ERROR_RETURN;
      }

      if ((sp = strstr(szTemp, "<td>Moon Set</td>")) != NULL)
      {
        if (!GetValue(sp, current.moonset, "<")) ERROR_RETURN;
      }
    }

    if (!bErr)
    {
      time_t now;
      struct tm *local;
      int    WINDSPEED;
      char   szTemp[256];

      //calculate current windchill
      strcpy(szTemp, current.wind);
      if (_strnicmp(szTemp, "Calm", 4))
      {
        char* s = strtok(szTemp, " ");
        strtok(NULL, " ");
        s = strtok(NULL, " ");

        if (s) WINDSPEED = atoi(s);
        else WINDSPEED = 0;
      }
      else WINDSPEED = 0;
      current.windchill = WINDSPEED < 4 ? current.temp :
           (int)(0.0817 *
           (3.71 * sqrt(WINDSPEED) - 0.25 * WINDSPEED + 5.81) *
           (current.temp - 91.4) +
           91.4);

      //calculate current.heatindex
      //many stations do not provide this information
      //so I'll do it myself
      if (current.humidity == 0 || current.temp < 60)  //a reasonable minimum!
        current.heatindex = current.temp;
      else
        current.heatindex = (int)(-42.379 +
           2.04901523 * current.temp +
          10.14333127 * current.humidity -
           0.22475541 * current.temp * current.humidity -
           0.00683783 * current.temp * current.temp -
           0.05481717 * current.humidity * current.humidity +
           0.00122874 * current.temp * current.temp * current.humidity +
           0.00085282 * current.temp * current.humidity * current.humidity -
           0.00000199 * current.temp * current.temp * current.humidity * current.humidity);

      //now check if we have a new extreme temperature (or new day)
      time(&now);
      local = localtime(&now); //military format
      if (local->tm_mday != tempMax.day || current.temp > tempMax.temp)
      {
        //new max temp
        tempMax.day  = local->tm_mday;
        tempMax.hour = local->tm_hour;
        tempMax.min  = local->tm_min;
        tempMax.temp = current.temp;
        WritePrivateProfileInt("Main", "MaxTemp", tempMax.temp, ini);
        WritePrivateProfileInt("Main", "MaxHour", tempMax.hour, ini);
        WritePrivateProfileInt("Main", "MaxMin",  tempMax.min, ini);
        WritePrivateProfileInt("Main", "MaxDay",  tempMax.day, ini);
      }
      if (local->tm_mday != tempMin.day || current.temp < tempMin.temp)
      {
        //new min temp
        tempMin.day  = local->tm_mday;
        tempMin.hour = local->tm_hour;
        tempMin.min  = local->tm_min;
        tempMin.temp = current.temp;
        WritePrivateProfileInt("Main", "MinTemp", tempMin.temp, ini);
        WritePrivateProfileInt("Main", "MinHour", tempMin.hour, ini);
        WritePrivateProfileInt("Main", "MinMin",  tempMin.min, ini);
        WritePrivateProfileInt("Main", "MinDay",  tempMin.day, ini);
      } //end if new min

      bChecking = FALSE;
      InvalidateRect(hMainWnd, NULL, TRUE);
    }
  }
  __except(1)
  {
    bErr = TRUE;
  }
}


/*** GetValue ***/
BOOL GetValue(const char* pointer, char* dest, char* delim)
{
  __try
  {
    char* start;
    char* end;

    start = strstr(pointer, "<b>");
    if (start)
    {
      start += 3;
      end = strstr(start, delim);
      strncpy(dest, start, end ? (int)(end-start) : sizeof(dest));
    }
    else
      return FALSE;
    return TRUE;
  }
  __except(1)
  {
    bErr=TRUE;
    return FALSE;
  }
}


/*** GetWind ***/
void GetWind(const char* pointer, char* dest)
{
  __try
  {
    char type[45] = "";
    char mph[5] = "";
    char* start;
    char* end;

    start = strstr(pointer, "<b>") + 3;
    end = strstr(start, "<");
    strncpy(type, start, end ? (int)(end-start) : sizeof(type));

    if (_stricmp(type, "calm"))
    {
      start = strstr(end, "<b>") + 3;
      //if (eUseMethod == useStation) start++; //must be 12 for global station
      end = strstr(start, "<");
      strncpy(mph, start, end ? (int)(end-start) : sizeof(mph));
      sprintf(dest, "%s at %s mph", type, mph);
    }
    else sprintf(dest, "Calm");
  }
  __except(1)
  {
    bErr=TRUE;
    return;
  }
}
